
note
	
	description: "Action that deletes view elements that are cuts that %
		%intersect the cut die of any select project"
	author: "David Hollenberg"
	date: "November 30, 2006"

class DELETE_INTERSECTING_CUT_ACTION
inherit
	VIEW_ELEMENT_ACTION
	GRIDABLE

feature -- Properties

	cut_dies: LIST [CONST_RECTANGLE]
			-- Cut dies of all selected projects

feature -- Modification

	set_cut_dies (a_list: like cut_dies)
			-- Set `cut_dies' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			cut_dies := a_list
		ensure
			cut_dies_set: cut_dies = a_list
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		do
			if an_element.is_cut and intersects_any_cut_die (an_element) then
				an_element.view.delete_cut (an_element)
				an_element.set_is_selected (False)
				an_element.add_to_changed_region
				an_element.add_change (create {CUT_DELETION_CHANGE}.make (an_element))
			else
				an_element.set_selected (an_element.is_selected, True)
			end
		end

feature {NONE} -- Implementation

	intersects_any_cut_die (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Does `an_element' intersect any cut die in
			-- `cut_dies'?
		local
			cen, low, high: DOUBLE
			cut_die: CONST_RECTANGLE
		do
			cen := an_element.gridded_cut_position
			from
				cut_dies.start
			until
				cut_dies.after or Result
			loop
				cut_die := cut_dies.item
				if an_element.is_horizontal_cut then
					low := grid.rounded_center (cut_die.low_y)
					high := grid.rounded_center (cut_die.high_y)
				else
					low := grid.rounded_center (cut_die.low_x)
					high := grid.rounded_center (cut_die.high_x)
				end
				Result := cen > low and cen < high
				cut_dies.forth
			end
		end

end
