
note
	
	description: "Action that extends both ends of a scribe or all sides of %
		% a partition to the first meetable edge if not already at a %
		% meetable edge, but without extending a scribe past an unmeetable edge"
	author: "David Hollenberg"
	date: "January 17, 2007"

class EXTEND_ACTION
inherit
	SCRIBE_EDGE_ACTION
		undefine
			default_create
		end
	SIDE_RESIZE_ACTION
		export 
			{NONE} resize_one_side
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			precursor {SIDE_RESIZE_ACTION}
			create resize_value.make_xy (0.0, 0.0)
		end

feature -- Properties

	changed: BOOLEAN
			-- Were any scribes extended since last call to
			-- `initialize'?

	is_horizontal_resize: BOOLEAN
			-- Should horizontal resizing be done?

	is_vertical_resize: BOOLEAN
			-- Should vertical resizing be done?

	is_meetable_only_requested: BOOLEAN
			-- Should only meetable edges be
			-- considered when looking for the closest
			-- edge to extend to?

	selected_only: BOOLEAN
			-- Are just selected scribes being extended, rather
			-- than all scribes

feature -- Modification

	set_is_horizontal_resize (a_value: like is_horizontal_resize)
			-- Set `is_horizontal_resize' to `a_value'
		do
			is_horizontal_resize := a_value
		ensure
			is_horizontal_resize_set: is_horizontal_resize = a_value
		end

	set_is_vertical_resize (a_value: like is_vertical_resize)
			-- Set `is_vertical_resize' to `a_value'
		do
			is_vertical_resize := a_value
		ensure
			is_vertical_resize_set: is_vertical_resize = a_value
		end

	set_is_meetable_only_requested (a_value: like is_meetable_only_requested)
			-- Set `is_meetable_only_requested' to `a_value'
		do
			is_meetable_only_requested := a_value
		ensure
			is_meetable_only_requested_set: is_meetable_only_requested = a_value
		end

	set_selected_only (a_value: like selected_only)
			-- Set `selected_only' to `a_value'
		do
			selected_only := a_value
		ensure
			is_selected_only_set: selected_only = a_value
		end

	initialize
			-- Initialize `Current'
		do
			changed := False
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		do
			if (not selected_only or an_element.is_selected) and an_element.intersects_any_field then
				-- Try to extend each end, if any part of
				-- it falls truly inside a field
				
				any_side_extended := False
				any_extend_failed := False
				create {ARRAYED_LIST [RETICLE_SIMPLE_CHANGE]} change_list.make (4)
				
				is_resized := False
				if is_horizontal_resize then
					extend_in_horizontal_direction (an_element, True, is_meetable_only_requested)
					update_state
					extend_in_horizontal_direction (an_element, False, is_meetable_only_requested)
					update_state
				end
				if is_vertical_resize then
					extend_in_vertical_direction (an_element, True, is_meetable_only_requested)
					update_state
					extend_in_vertical_direction (an_element, False, is_meetable_only_requested)
					update_state
				end
				if is_resized then
					add_change (an_element)
					an_element.add_old_and_new_to_changed_region
				end
				
				if any_side_extended then
					if an_element.is_scribe then
						an_element.view.changed_scribes.extend (an_element)
					else
						an_element.view.changed_partitions.extend (an_element)
					end
				end
				if any_extend_failed then
					if an_element.is_scribe then
						an_element.view.locked_scribes.extend (an_element)
					else
						an_element.view.locked_partitions.extend (an_element)
					end
				end
			end
		end

feature {NONE} -- Implementation

	extend_in_horizontal_direction (an_element: RETICLE_VIEW_ELEMENT is_low_end, meetable_only: BOOLEAN)
			-- Extend one side of `an_element' in
			-- horizontal direction.  Extend left if
			-- `is_low_end', otherwise extend right end.
			-- Do nothing if no part of end is truly
			-- inside a field of view.  If `meetable_only'
			-- then ignore non-meetable edges when looking
			-- for the closest edge
		do
		end

	extend_in_vertical_direction (an_element: RETICLE_VIEW_ELEMENT is_low_end, meetable_only: BOOLEAN)
			-- Extend one end of `an_element' in vertical
			-- direction.  Extend bottom end if
			-- `is_low_end', otherwise extend top end.  Do
			-- nothing if no part of end is truly inside a
			-- field of view.  If `meetable_only' then
			-- ignore non-meetable edges when looking for
			-- the closest edge
		do
		end

	update_state
			-- Update `any_side_extended'
			-- and `any_extend_failed' after call to
			-- `extend_in_horizontal_direction' or
			-- `extend_in_vertical_direction'
		do
			any_side_extended := any_side_extended or was_extended
			any_extend_failed := any_extend_failed or needs_extend and not was_extended
		end
	
	any_side_extended: BOOLEAN
			-- Were any sides of last element extended?
			-- (might not be extended even if needed if
			-- locked in the "extend" dimension)

	any_extend_failed: BOOLEAN
			-- Did any extend fail due to "locked in
			-- extend dimension" issue?

	needs_extend: BOOLEAN
			-- Did last element passed to
			-- `extend_in_horizontal_direction' or
			-- `extend_in_vertical_direction' need to be
			-- extended (though it might not have been
			-- extended if it was locked in extend
			-- dimension)?

	was_extended: BOOLEAN
			-- Was last element passed to
			-- `extend_in_horizontal_direction' or
			-- `extend_in_vertical_direction' extended?

	resize_value: POINT
			-- Point that specifies how much scribe was resized,
			-- if `was_extended'

end
