
note
	
	description: "Action that replaces a scribe that intersects more %
		%than one field of view with a set of scribes that %
		%represents the intersection of the scribe with each field"
	author: "David Hollenberg"
	date: "October 31, 2006"

class SEPARATE_SCRIBE_ACTION
inherit
	MULTIPLE_CHANGE_ACTION

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			sep: BOOLEAN
		do
			sep := False
				if an_element.is_scribe and then an_element.intersects_more_than_one_field then
					if an_element.is_horizontal_scribe then
						if an_element.is_movable_in_x then
							an_element.view.changed_scribes.extend (an_element)
							separate_into_fields (an_element)
							sep := True
						else
							an_element.view.locked_scribes.extend (an_element)
						end
					elseif an_element.is_vertical_scribe then
						if an_element.is_movable_in_y then
							an_element.view.changed_scribes.extend (an_element)
							separate_into_fields (an_element)
							sep := True
						else
							an_element.view.locked_scribes.extend (an_element)
						end
					end
				end
			if not sep then
				an_element.set_selected (an_element.is_selected, True)
			end
		end

feature {NONE} -- Implementation

	separate_into_fields (an_element: RETICLE_VIEW_ELEMENT)
			-- Delete `an_element' and add one new scribe for
			-- each field it intersects
		local
			rects: LIST [RECTANGLE]
			rect: RECTANGLE
			new_scribe: RETICLE_VIEW_ELEMENT
		do
			rects := an_element.intersection_with_fields
			create {ARRAYED_LIST [RETICLE_SIMPLE_CHANGE]} change_list.make (rects.count + 1)

			-- Delete old scribe
			
			an_element.view.deleted_elements.extend (an_element)
			an_element.add_to_changed_region
			change_list.extend (create {DIE_FULL_DELETION_CHANGE}.make (an_element))
			
			-- Add one new scribe for each field old scribe
			-- intersects

			from
				rects.start
			until
				rects.after
			loop
				rect := rects.item
				if an_element.is_horizontal_scribe then
				else
				end
				new_scribe := an_element.view.new_scribe (Void, an_element)
				-- No need to add `new_scribe' to view since
				-- setting its selection status added it to
				-- appropriate list
				change_list.extend (create {DIE_CREATION_CHANGE}.make (new_scribe))
				rects.forth
			end
			add_change (an_element)
		end

end
