
note
	
	description: "Action that trims a scribe or partition that intersects at least one %
		%field of view and that has an end that is not in any field %
		%of view by trimming that end back to the field of view edge"
	author: "David Hollenberg"
	date: "October 31, 2006"

class TRIM_ACTION
inherit
	ANY
		undefine
			default_create
		end
	SIDE_RESIZE_ACTION
		export 
			{NONE} resize_one_side
		end

feature -- Properties

	is_scribe_trim_requested: BOOLEAN
			-- Should scribes be trimmed?

	is_partition_trim_requested: BOOLEAN
			-- Should partitions be trimmed?

feature -- Modification

	set_is_scribe_trim_requested (a_value: like is_scribe_trim_requested)
		do
			is_scribe_trim_requested := a_value
		end

	set_is_partition_trim_requested (a_value: like is_partition_trim_requested)
		do
			is_partition_trim_requested := a_value
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			left_outside, right_outside, top_outside, bottom_outside: BOOLEAN
			left, right, top, bottom: DOUBLE
			dist_left, dist_right, dist_up, dist_down: DOUBLE
		do
			if ((an_element.is_scribe and is_scribe_trim_requested) or (an_element.is_partition and is_partition_trim_requested)) and then an_element.intersects_any_field then
				left := grid.rounded_edge (an_element.left_x)
				right := grid.rounded_edge (an_element.right_x)
				top := grid.rounded_edge (an_element.top_y)
				bottom := grid.rounded_edge (an_element.bottom_y)
				dist_left := an_element.closest_field_edge_to_left (right) - right
				dist_right := an_element.closest_field_edge_to_right (left) - left
				dist_up := an_element.closest_field_edge_above (bottom) - bottom
				dist_down := an_element.closest_field_edge_below (top) - top
				left_outside := an_element.x_outside_all_fields (left) and dist_right /= 0.0
				right_outside := an_element.x_outside_all_fields (right) and dist_left /= 0.0
				top_outside := an_element.y_outside_all_fields (top) and dist_down /= 0.0
				bottom_outside := an_element.y_outside_all_fields (bottom) and dist_up /= 0.0
				
				is_resized := False
				create {ARRAYED_LIST [RETICLE_SIMPLE_CHANGE]} change_list.make (2)
				if an_element.is_horizontal_scribe and (left_outside or right_outside) then
					if an_element.is_movable_in_x then
						if left_outside then
							resize_one_side (an_element, True, False, False, False, dist_right, 0.0)
						end
						if right_outside then
							resize_one_side (an_element, False, True, False, False, dist_left, 0.0)
						end
						an_element.view.changed_scribes.extend (an_element)
					else
						an_element.view.locked_scribes.extend (an_element)
					end
				elseif an_element.is_vertical_scribe and (top_outside or bottom_outside) then
					if an_element.is_movable_in_y then
						if bottom_outside then
							resize_one_side (an_element, False, False, True, False, 0.0, dist_up)
						end
						if top_outside then
							resize_one_side (an_element, False, False, False, True, 0.0, dist_down)
						end
						an_element.view.changed_scribes.extend (an_element)
					else
						an_element.view.locked_scribes.extend (an_element)
					end
				elseif an_element.is_partition and (left_outside or right_outside or top_outside or bottom_outside) then
					if (not an_element.is_movable_in_x and (left_outside or right_outside) or
					    not an_element.is_movable_in_y and (top_outside or bottom_outside)) then
						an_element.view.locked_partitions.extend (an_element)
					end
					if an_element.is_movable_in_x then
						if left_outside then
							resize_one_side (an_element, True, False, False, False, dist_right, 0.0)
						end
						if right_outside then
							resize_one_side (an_element, False, True, False, False, dist_left, 0.0)
						end
					end
					if an_element.is_movable_in_y then
						if bottom_outside then
							resize_one_side (an_element, False, False, True, False, 0.0, dist_up)
						end
						if top_outside then
							resize_one_side (an_element, False, False, False, True, 0.0, dist_down)
						end
					end
					if is_resized then
						an_element.view.changed_partitions.extend (an_element)
					end
				end
				add_change (an_element)
				if is_resized then
					an_element.add_old_and_new_to_changed_region
				end
			end
			if an_element.is_selected then
				an_element.include_in_selection_bounding_box
			end
		end

end
