
note
	
	description: "Action that unoverlaps scribe with all or selected %
		%scribes of opposite orientation that intersect it"
	author: "David Hollenberg"
	date: "January 17, 2007"

class UNOVERLAP_SCRIBE_ACTION
inherit
	SCRIBE_EDGE_ACTION
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			create resize_amount.make_xy (0, 0)
		end

feature -- Properties

	vertical_scribes: LIST [RETICLE_VIEW_ELEMENT]
			-- Vertical scribes that participate in Unoverlap
			-- operation

feature -- Modification

	set_vertical_scribes (a_list: like vertical_scribes)
			-- Set `vertical_scribes' to `a_list'
		do
			vertical_scribes := a_list
		ensure
			vertical_scribes_set: vertical_scribes = a_list
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		local
			vscribe: RETICLE_VIEW_ELEMENT
		do
			from
				vertical_scribes.start
			until
				vertical_scribes.after or an_element.is_marked
					-- Until last vertical scribe or until
					-- horizontal scribe marked for deleted
			loop
				vscribe := vertical_scribes.item
				if an_element.intersects (vscribe) then
					unoverlap_one_scribe (an_element, vscribe)
				end
				vertical_scribes.forth
			end
		end

feature {NONE} -- Implementation

	unoverlap_one_scribe (hscribe, vscribe: RETICLE_VIEW_ELEMENT)
			-- Unoverlap `hscribe' and `vscribe'
		local
			horiz_left, horiz_right, horiz_bottom, horiz_top: DOUBLE
			vert_left, vert_right, vert_bottom, vert_top: DOUBLE
			horiz_encloses_vert, vert_encloses_horiz: BOOLEAN
			view: RETICLE_VIEW
		do
			view := hscribe.view

			horiz_left := grid.rounded_edge (hscribe.left_x)
			horiz_right := grid.rounded_edge (hscribe.right_x)
			horiz_bottom := grid.rounded_edge (hscribe.bottom_y)
			horiz_top := grid.rounded_edge (hscribe.top_y)
			
			vert_left := grid.rounded_edge (vscribe.left_x)
			vert_right := grid.rounded_edge (vscribe.right_x)
			vert_bottom := grid.rounded_edge (vscribe.bottom_y)
			vert_top := grid.rounded_edge (vscribe.top_y)
		
			horiz_encloses_vert := hscribe.encloses (vscribe)
			vert_encloses_horiz := vscribe.encloses (hscribe)
			if horiz_encloses_vert and vert_encloses_horiz then
				remove_scribe (vscribe)
			elseif horiz_encloses_vert then
				remove_scribe (vscribe)
			elseif vert_encloses_horiz then
				remove_scribe (hscribe)
			else
				-- Neither scribe encloses the other
				if horiz_left < vert_left and horiz_right <= vert_right and horiz_top <= vert_top and horiz_bottom >= vert_bottom then
					-- Tee rotated 90 clockwise
					resize_scribe_with_change (hscribe, vert_left - horiz_right, 0, False)
					
				elseif horiz_right > vert_right and horiz_left >= vert_left and horiz_top <= vert_top and horiz_bottom >= vert_bottom then
					-- Tee rotated 90 counter-clockwise
					resize_scribe_with_change (hscribe, vert_right - horiz_left, 0, True)
				elseif vert_top > horiz_top and vert_bottom >= horiz_bottom and vert_left >= horiz_left and vert_right <= horiz_right then
					-- Tee rotated 180
					resize_scribe_with_change (vscribe, 0, horiz_top - vert_bottom, True)
				elseif vert_bottom < horiz_bottom and vert_top <= horiz_top and vert_left >= horiz_left and vert_right <= horiz_right then
					-- Upright tee
					resize_scribe_with_change (vscribe, 0, horiz_bottom - vert_top, False)
				elseif horiz_left < vert_left and horiz_right > vert_right and vert_top > horiz_top and vert_bottom < horiz_bottom then
					-- Cross.  Decide whether to break
					-- horizontal or vertical scribe.
					-- Prefer the one that would leave
					-- a fragment or break vertical one if
					-- neither would leave a fragment.

					horizontal_fragment_count := 0
					vertical_fragment_count := 0

					
					if horizontal_fragment_count > vertical_fragment_count then
						-- Add new horizontal scribe to left
						-- of vertical scribe and resize left
						-- edge of existing one
						-- to right edge of vertical scribe

						break_scribe (hscribe, horiz_left, horiz_bottom, vert_left, horiz_top, True, vert_right - horiz_left, 0, True)
					else
						-- Add new vertical scribe below
						-- horizontal scribe and resize bottom
						-- edge of existing one to top edge of
						-- horizontal scribe
					
						break_scribe (vscribe, vert_left, vert_bottom, vert_right, horiz_bottom, False, 0, horiz_top - vert_bottom, True)
					end
				else
					-- Remaining cases are either impossible
					-- or should not result in any action
					-- because cannot unoverlap without
					-- changing area covered by scribes
				end
			end
		end

	resize_scribe_with_change (a_scribe: RETICLE_VIEW_ELEMENT a_x, a_y: DOUBLE is_left_or_bottom: BOOLEAN)
			-- Resize `a_scribe' end indicated by
			-- `is_left_or_bottom' by (`a_x', `a_y').  Add a
			-- simple change to the history list if resized
		do
			resize_scribe (a_scribe, a_x, a_y, is_left_or_bottom)
			if last_change /= Void then
				a_scribe.add_change (last_change)
			end
		end
						
	resize_scribe (a_scribe: RETICLE_VIEW_ELEMENT a_x, a_y: DOUBLE is_left_or_bottom: BOOLEAN)
			-- Resize `a_scribe' end indicated by
			-- `is_left_or_bottom' by (`a_x', `a_y').
			-- Set `last_change' to change made or to Void if
			-- no change
		do
			if a_scribe.is_movable_in_length_dimension then
				resize_amount.make_xy (a_x, a_y)
				a_scribe.resize_scribe_by (resize_amount, is_left_or_bottom)
				a_scribe.set_view_coordinates
				create {SCRIBE_END_POSITION_CHANGE} last_change.make (a_scribe, resize_amount.x, resize_amount.y, is_left_or_bottom)
			else
				last_change := Void
			end
		end
						
	remove_scribe (a_scribe: RETICLE_VIEW_ELEMENT)
			-- Remove `a_scribe'
		do
			if a_scribe.is_deletable then
				a_scribe.view.deleted_elements.extend (a_scribe)
				a_scribe.mark
				a_scribe.add_change (create {DIE_FULL_DELETION_CHANGE}.make (a_scribe))
			else
			end
		end

	break_scribe (a_scribe: RETICLE_VIEW_ELEMENT ll, lr, ul, ur: DOUBLE is_horiz: BOOLEAN a_resize_x, a_resize_y: DOUBLE is_left_or_bottom: BOOLEAN)
			-- Create new scribe defined by lower left corner
			-- (`ll', `lr') and upper right corner (`ul', `ur')
			-- with properties of `a_scribe'.  Resize `a_scribe'
			-- by (`a_resize_x', `a_resize_y') with
			-- `is_left_or_bottom' indicating which end to resize
		local
			rect: RECTANGLE
			new_scribe: RETICLE_VIEW_ELEMENT
			view: RETICLE_VIEW
			list: LIST [RETICLE_SIMPLE_CHANGE]
		do
			view := a_scribe.view
			if a_scribe.is_movable_in_length_dimension then
				create rect.make_by_corners (ll, lr, ul, ur)
				if is_horiz then
				else
				end
				new_scribe := view.new_scribe (Void, a_scribe)
				view.new_elements.extend (new_scribe)
				resize_scribe (a_scribe, a_resize_x, a_resize_y, is_left_or_bottom)
				create {ARRAYED_LIST [RETICLE_SIMPLE_CHANGE]} list.make (2)
				list.extend (create {DIE_CREATION_CHANGE}.make (new_scribe))
				list.extend (last_change)
					-- From resize of existing scribe
				a_scribe.add_change (create {RETICLE_SIMPLE_CHANGE_LIST}.make (a_scribe, list))
			else
			end
		end
	
	last_change: RETICLE_SIMPLE_CHANGE
			-- Last change made by `resize_scribe'

	resize_amount: POINT
			-- Point that specifies how much to resize scribe

	horizontal_fragment_count: INTEGER
			-- Number of horizontal scribe ends that appear
			-- to be fragments, for scribe intersection that
			-- forms a cross
	
	vertical_fragment_count: INTEGER
			-- Number of vertical scribe ends that appear
			-- to be fragments, for scribe intersection that
			-- forms a cross
	
end
