
note
	
	description: "Action to move elements by a computed distance"
	author: "David Hollenberg"
	date: "November 16, 2006"

deferred class VIEW_ELEMENT_MOVE_ACTION
inherit
	VIEW_ELEMENT_ACTION
		redefine
			default_create
		end
	VIEW_ELEMENT_ACTIONS
		export
			{NONE} all
		redefine
			default_create
		end
	GRIDABLE
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			create new_center.make_xy (0, 0)
			create move_distance.make_xy (0, 0)
		end

feature -- Properties

	is_valid: BOOLEAN
			-- Do all elements checked so far have valid locations
			-- (that will fit in database)?

	is_validate_only: BOOLEAN
			-- Should location validity checks be applied without
			-- moving anything?

feature -- Modification

	set_valid (a_value: like is_valid)
			-- Set `is_valid' to `a_value'
		do
			is_valid := a_value
		ensure
			is_valid_set: is_valid = a_value
		end

	set_validate_only (a_value: like is_validate_only)
			-- Set `is_validate_only' to `a_value'
		do
			is_validate_only := a_value
		ensure
			is_validate_only_set: is_validate_only = a_value
		end

feature -- Operations

	apply (an_element: RETICLE_VIEW_ELEMENT)
			-- Apply action to `an_element'
		do
			compute_center (an_element)
			if not an_element.view.center_fits_in_database (new_center) then
				is_valid := False
			end
			compute_move_distance (an_element)
			if is_validate_only then
				do_additional_validation (an_element)
			else
				move_action.set_move_distance (move_distance)
				move_action.apply (an_element)
			end
		end

feature {NONE} -- Implementation

	compute_center (an_element: RETICLE_VIEW_ELEMENT)
			-- Compute `new_center' for `an_element'
		deferred
		end

	do_additional_validation (an_element: RETICLE_VIEW_ELEMENT)
			-- Do any additional validation needed
		deferred
		end

	compute_move_distance (an_element: RETICLE_VIEW_ELEMENT)
			-- Compute `move_distance' for `an_element'
		deferred
		end

	new_center: POINT
			-- New center for `an_element'

	move_distance: POINT
			-- Distance to move `an_element' to make its
			-- center be `new_center'

end
