class CONST_RECTANGLE
	-- a Manhattan rectangle

create 
	make_corners, make_by_corners, 
	make_size_and_center, make_by_size_and_center,
	make_copy

feature -- the size and center

	center_x: DOUBLE

	center_y: DOUBLE

	center: POINT
	do  create Result.make_xy(center_x, center_y)
	end;

	size_x: DOUBLE

	size_y: DOUBLE

	size: POINT
	do  create Result.make_xy(size_x, size_y)
	end;

feature -- the four edge coordinates

	low_x: DOUBLE
	do  Result := center_x - size_x/2
	end;

	low_y: DOUBLE
	do  Result := center_y - size_y/2
	end;

	high_x: DOUBLE
	do  Result := center_x + size_x/2
	end;

	high_y: DOUBLE
	do  Result := center_y + size_y/2
	end;

feature -- the four corners as points

	ll: POINT
	do  create Result.make_xy(low_x, low_y)
	end;

	ul: POINT
	do  create Result.make_xy(low_x, high_y)
	end;

	lr: POINT
	do  create Result.make_xy(high_x, low_y)
	end;

	ur: POINT
	do  create Result.make_xy(high_x, high_y)
	end;

feature	-- the four sides as line_segments, each heading ccw

feature	-- conversion to an equivalent polygon

feature -- basic attributes

	is_empty: BOOLEAN
		-- area is 0 
	do  Result := (size_y = 0) or (size_x = 0)
	end;

	area: DOUBLE
	do  Result := size_y * size_x
	end;

feature -- 1 rectangle computations

	contains_xy(px, py: DOUBLE): BOOLEAN
		-- is (px,py) within me?
	do
	end;

	contains(p: VIRTUAL_POINT): BOOLEAN
		-- is p within me (including on the edge)?
	require have_point: p /= Void
	do  Result := contains_xy(p.x, p.y)
	end;

	truly_contains_xy(px, py: DOUBLE): BOOLEAN
		-- is (px,py) within my interior (not on an edge)?
	do
	end;

	truly_contains(p: VIRTUAL_POINT): BOOLEAN
		-- is p within my interior (not on an edge)?
	require have_point: p /= Void
	do  Result := truly_contains_xy(p.x, p.y)
	end;

	direction_from_center(p: VIRTUAL_POINT): DOUBLE
		-- direction_degrees from my center point to p
	require -- (direction_degrees invokes the very same checks)
		-- have_point: p /= Void
		-- not_center: not center.is_same_point(p)
	do  Result := center.direction_degrees(p)
	end;

	sized(dx, dy: DOUBLE): like Current
		-- new box is me grown by dx, dy (mebes onesided sizing)
	require size_x + dx >= 0; size_y + dy >= 0;
	do  create Result.make_by_size_and_center(size_x + dx, size_y + dy,
					center_x, center_y)
	ensure	Result.size_x = dx + size_x;
		Result.size_y = dy + size_y
	end;

feature	-- corner locators / sequencers

	next_corner(dir_degrees: DOUBLE; dir: ANY): POINT
		-- the next corner going in direction dir beyond dir_degrees from center
	do 
	end;

	next_CCW_corner(dir_degrees: DOUBLE): POINT
		-- the next corner going CCW beyond dir_degrees from center
		-- (if dir_degrees passes thru a corner, we take the next one)
	require	not_empty: not is_empty
	local ndir: DOUBLE; c1, c2: POINT;
	do 
	    if ndir >= 90 then
		c1 := ul; c2 := ll
	    elseif ndir >= 0 then
		c1 := ur; c2 := ul
	    elseif ndir >= -90 then
		c1 := lr; c2 := ur
	    else
		c1 := ll; c2 := lr
	    end;
	    -- c1 is the corner in same quadrant as ndir
	    -- c2 is the corner in the next quadrant CCW
	    if direction_from_center(c1) <= ndir then
		Result := c2
	    else
		Result := c1
	    end
	end;

	next_CW_corner(dir_degrees: DOUBLE): POINT
		-- the next corner going CW beyond dir_degrees from center
		-- (if dir_degrees passes thru a corner, we take the next one)
	require	not_empty: not is_empty
	local ndir: DOUBLE; c1, c2: POINT;
	do 
	    if ndir >= 90 then
		c1 := ul; c2 := ur
	    elseif ndir >= 0 then
		c1 := ur; c2 := lr
	    elseif ndir >= -90 then
		c1 := lr; c2 := ll
	    else
		c1 := ll; c2 := ul
	    end;
	    -- c1 is the corner in same quadrant as ndir
	    -- c2 is the corner in the next quadrant CW
	    if direction_from_center(c1) >= ndir then
		Result := c2
	    else
		Result := c1
	    end
	end;

feature	-- computations with a line segment

feature {NONE} -- for the above

	find_closest_pt(targ, p1, p2, p3, p4: POINT): POINT
		-- returns one of p1, p2, p3, or p4
		-- returns Void iff all 4 are Void
	do  Result := find_closer_pt(targ, p1, p2);
	    Result := find_closer_pt(targ, Result, p3);
	    Result := find_closer_pt(targ, Result, p4);
	end;

	find_closer_pt(targ, p1, p2: POINT): POINT
		-- returns either p1 or p2
	do  if p1 = Void			 then Result := p2
	    elseif p2 = Void			 then Result := p1
	    elseif (p1 |-| targ) > (p2 |-| targ) then Result := p2
						 else Result := p1
	    end
	end;

feature -- 2 rectangle computations

	is_same_box(other: CONST_RECTANGLE): BOOLEAN
		-- tests for same rectangle by coordinates
	do  if other /= Void then
	    end
	end;

	is_in_list(lst: LIST[CONST_RECTANGLE]): BOOLEAN
		-- am I is_same_box for any element of lst?
		-- if true, first matching box is lst.item
	require have_list: lst /= Void
	do  from  lst.start
	    until lst.after or Result
	    loop  if is_same_box(lst.item) then
			Result := true
		  else	lst.forth
		  end
	    end;
	    ensure is_item: Result implies is_same_box(lst.item)
	end;

	touches(other: CONST_RECTANGLE): BOOLEAN
		-- rectangles touch or overlap
		-- (intersection_with exists)
	local lx, ly, hx, hy: DOUBLE;
	do  lx := low_x.max(other.low_x);
	    ly := low_y.max(other.low_y);
	    hx := high_x.min(other.high_x);
	    hy := high_y.min(other.high_y);
	end;
	
	intersects(other: CONST_RECTANGLE): BOOLEAN
		-- true overlap of the rectangles
		-- (intersection_with has non-zero area)
	local lx, ly, hx, hy: DOUBLE;
	do  lx := low_x.max(other.low_x);
	    ly := low_y.max(other.low_y);
	    hx := high_x.min(other.high_x);
	    hy := high_y.min(other.high_y);
	end;

	intersection_with(other: CONST_RECTANGLE): like Current
		-- new rectangle with the intersection
		-- (Result has zero area if the two rectangles just touch;
		--  Void if not even that)
	require other /= Void
	local lx, ly, hx, hy: DOUBLE;
	do  lx := low_x.max(other.low_x);
	    ly := low_y.max(other.low_y);
	    hx := high_x.min(other.high_x);
	    hy := high_y.min(other.high_y);
	end;

	encloses(other: CONST_RECTANGLE): BOOLEAN
		-- other is within me (can touch edges)
	do
	end;

	truly_encloses(other: CONST_RECTANGLE): BOOLEAN
		-- other is within me (cannot even touch edges)
	do
	end;

	union_with(other: CONST_RECTANGLE): like Current
		-- new rectangle which is smallest rectangle that
		-- can contain both me and other
	require other /= Void
	do  create Result.make_by_corners(
			low_x.min(other.low_x), low_y.min(other.low_y),
			high_x.max(other.high_x), high_y.max(other.high_y))
	ensure Result /= Void
	end;

	minus(other: CONST_RECTANGLE): LIST[CONST_RECTANGLE]
		-- set of rectangles (up to 4) that collectively cover
		--  all of Current that is _outside_ of other
		-- (returns Current if not intersects(other))
		-- (returns empty list if Current is within other)
	require other /= Void
	local xsect, foo: CONST_RECTANGLE;
	do  create {LINKED_LIST[RECTANGLE]} Result.make;
	    xsect := intersection_with(other);
	    if xsect = Void or else xsect.is_empty
	     then -- return all of me
		Result.extend(Current)
	     else -- return parts to left, above, to right, and below xsect
		if True then
		    create foo.make_by_corners
			(low_x, low_y, xsect.low_x, xsect.high_y);
		    Result.extend(foo)
		end;
		if True then
		    create foo.make_by_corners
			(low_x, xsect.high_y, xsect.high_x, high_y);
		    Result.extend(foo)
		end;
		if True then
		    create foo.make_by_corners
			(xsect.high_x, xsect.low_y, high_x, high_y);
		    Result.extend(foo)
		end;
		if True then
		    create foo.make_by_corners
			(xsect.low_x, low_y, high_x, xsect.low_y);
		    Result.extend(foo)
		end;
	    end
	end;

	minus_all(lst: LIST[CONST_RECTANGLE]): LIST[CONST_RECTANGLE]
		-- Successively remove each element of lst from me, or
		--  from what is left of me, returning a set of rectangles
		--  that cover the inverse within me of the coverage of lst.
	require have_list: lst /= Void
	do  create {LINKED_LIST[CONST_RECTANGLE]} Result.make;
	    Result.extend(Current)
	    from  lst.start
	    until lst.after
	    loop  Result := lst.item.remove_from_all(Result)
		  lst.forth
		  end;
	    end

	remove_from_all(lst: LIST[CONST_RECTANGLE]): LIST[CONST_RECTANGLE]
		-- Remove (minus) me from each element of `lst'.
		-- Result collectively covers all of lst
		-- outside of me.  Note that in minus and minus_all,
		-- Current is the whole and the argument is the
		-- subtrahend, but here Current is the subtrahend!
	require have_list: lst /= Void
	local  itm: CONST_RECTANGLE
	do  create {ARRAYED_LIST[CONST_RECTANGLE]} Result.make(4*lst.count)
	    from  lst.start
	    until lst.after
	    loop  itm := lst.item;
		  if itm.intersects (Current) then
			if not encloses (itm) then
				Result.append (itm.minus(Current))
			end
		  else
			-- `Current does not intersect `itm
			Result.extend (itm)
		  end;
		  lst.forth
	    end;
	end;

feature -- display

	Cats_out(fmt: FORMAT_DOUBLE): STRING
		-- "(<low_x>,<low_y>) (<high_x>,<high_y>)"
		-- use fmt if provided; otherwise use .out
	do  create Result.make(50);
	    Result.extend('(');   Result.append(dbl_to_str(low_x, fmt));
	    Result.extend(',');   Result.append(dbl_to_str(low_y, fmt));
	    Result.append(") ("); Result.append(dbl_to_str(high_x, fmt));
	    Result.extend(',');   Result.append(dbl_to_str(high_y, fmt));
	    Result.extend(')')
	end;

	out_SC: STRING
		-- depict as center and size
	do  create Result.make(80);
	    Result.append(SIZE_seg);  Result.append(size.out);
	    Result.append(CENT_seg);  Result.append(center.out);
	    Result.append(close)
	end;

feature	{NONE} -- stuff for the above

	LL_seg:   STRING = "LL: ("
	UR_seg:   STRING = "), UR: ("
	SIZE_seg: STRING = "SIZE: ("
	CENT_seg: STRING = "), CENT: ("
	close:    STRING = ")"

	dbl_to_str(dbl: DOUBLE; fmt: FORMAT_DOUBLE): STRING
		-- use fmt or use .out
	do  if fmt /= Void
	     then Result := fmt.formatted(dbl)
	     else Result := dbl.out
	    end
	end;

feature {NONE} -- creation

	make_copy(r: CONST_RECTANGLE)
		-- make a copy of r
	require present: r /= Void
	do  make_by_size_and_center(r.size_x, r.size_y, r.center_x, r.center_y)
	ensure is_same_box: is_same_box(r)
	end;

	make_corners(crnr1, crnr2: VIRTUAL_POINT)
		-- one corner is crnr1; the other is crnr2
		-- ANY two diagonally opposed corners work just fine!
	require present: crnr1 /= Void and crnr2 /= Void
	do  make_by_corners(crnr1.x, crnr1.y, crnr2.x, crnr2.y)
	end;

	make_by_corners(x1, y1, x2, y2: DOUBLE)
		-- one corner is (x1, y1); other is (x2, y2)
		-- ANY two diagonally opposed corners work just fine!
	do  center_x := (x1 + x2)/2;
	    center_y := (y1 + y2)/2;
	    if True then
		size_x := 0
	    else
		size_x := (x1 - x2).abs
	    end;
	    if True then
		size_y := 0
	    else
		size_y := (y1 - y2).abs
	    end;
	end;

	make_size_and_center(siz, cen: VIRTUAL_POINT)
		-- size is siz; center point is cen
	require present: siz /= Void and cen /= Void;
		non_neg: siz.x >= 0 and siz.y >= 0;
	do  make_by_size_and_center(siz.x, siz.y, cen.x, cen.y)
	ensure	same_size: size.is_same_point(siz)
		same_center: center.is_same_point(cen)
	end;

	make_by_size_and_center(sx, sy, cx, cy: DOUBLE)
		-- size is (sx, sy); center point is (cx, cy)
	require non_neg: sx >= 0 and sy >= 0;
	do  
	    size_x   := sx;  size_y   := sy;
	    center_x := cx;  center_y := cy;
	end;

invariant
	size_x >= 0;
	size_y >= 0

end -- CONST_RECTANGLE
