
note
	
	description: "Edge of element of reticle model"
	author: "David Hollenberg"
	date: "January 3, 2007"

class MODEL_ELEMENT_EDGE

feature -- Properties

	lower_end: DOUBLE
			-- Gridded lower endpoint of edge, in model coordinates

	upper_end: DOUBLE
			-- Gridded upper endpoint of edge, in model coordinates

	position: DOUBLE
			-- Gridded position of edge (X value for
			-- vertical edge, Y value for horizontal
			-- edge), in model coordinates

	has_position: BOOLEAN = True
			-- Is there a position?

	is_low_edge: BOOLEAN
			-- Is this the low edge of object (left or
			-- bottom edge)?

	is_meetable: BOOLEAN
			-- Can a scribe be extended to meet this edge
			-- (assuming it is the closest edge to end of
			-- scribe to be extended)?

	truly_intersects (low, high: DOUBLE): BOOLEAN
			-- Does segment (`low', `high') intersect edge,
			-- not including end points of edge?
		do
			Result := low < upper_end and high > lower_end
		end

	contains (low, high: DOUBLE): BOOLEAN
			-- Is segment (`low', `high') completely contained
			-- within edge (sub-segment, not necessarily proper)?
		do
			Result := low >= lower_end and high <= upper_end
		end

feature -- Modification

	set_ends (a_lower_end: like lower_end an_upper_end: like upper_end)
			-- Set `lower_end' to `a_lower_end' and `upper_end'
			-- to `an_upper_end'
		do
			lower_end := a_lower_end
			upper_end := an_upper_end
		ensure
			lower_end_set: lower_end = a_lower_end
			upper_end_set: upper_end = an_upper_end
		end

	set_position (a_value: like position)
			-- Set `position' to `a_value'
		do
			position := a_value
		ensure
			position_set: position = a_value
		end

	set_is_low_edge (a_value: like is_low_edge)
			-- Set `is_low_edge' to `a_value'
		do
			is_low_edge := a_value
		ensure
			is_low_edge_set: is_low_edge = a_value
		end

	set_is_meetable (a_value: like is_meetable)
			-- Set `is_meetable' to `a_value'
		do
			is_meetable := a_value
		ensure
			is_meetable_set: is_meetable = a_value
		end

end
