
note
	
	description: "Object that is an element of a view of a reticle"
	author: "David Hollenberg"
	date: "July 25, 2006"

class RETICLE_VIEW_ELEMENT
inherit
	VIEW_ELEMENT
		redefine
			view,anchored_view_sub_element
		end

feature {NONE} -- Creation

	make (a_view: like view)
		require
			a_view_not_void: a_view /= Void
		do
			view := a_view
			create {VIEW_ELEMENT_STANDARD_DRAWER} drawer
		end

feature -- Properties

	view: RETICLE_VIEW
			-- View of which this is one element

	max_db_value: DOUBLE
			-- Maximum value that can fit in the database
		do
		end

	grid: GRID
			-- Grid for object sizes and edges
		do
		end

	is_color_generator_changed: BOOLEAN
			-- Did color generator change on last call to
			-- to `set_color_generator'?

	is_hidden_element_displayed: BOOLEAN
			-- Is a hidden element displayed in current view?

	is_hidden_element_displayed_changed: BOOLEAN
			-- Did display (yes or no) of hidden element
			-- change on last call to
			-- `set_is_hidden_element_displayed'?

	line_width: INTEGER
			-- Width of line to draw for border, in pixels
		do
			if is_selected then
				Result := 3
			else
				Result := 1
			end
		end

	label: STRING
			-- Current label to display
		do
		end

	label_rotation: INTEGER
			-- Rotation of label in degrees clockwise
		do
			Result := rotation
		end

	is_interior_filled: BOOLEAN
			-- Should interior of element be filled when displayed?
		do
		end

	is_opaque: BOOLEAN
			-- Should element be displayed as opaque, so that
			-- it completely obscures the part of any element
			-- that is below it in stacking order?
		do
		end

	is_transparent: BOOLEAN
			-- Should element be displayed as transparent, so that
			-- we attempt to draw parts of elements that fall
			-- below it in stacking order so that they are visible?
		do
		end

	is_selected: BOOLEAN
			-- Is element currently selected?

	is_selected_changed: BOOLEAN
			-- Did last call to `set_is_selected' change the
			-- value of `is_selected'?

	included_in_changed_region: BOOLEAN
			-- Is `Current' already included in the region which
			-- has been changed by the current operation?

	matched_last_search: BOOLEAN
			-- Did `Current' match the search criteria for last
			-- Search operation?

	matched_last_search_changed: BOOLEAN
			-- Did value of `matched_last_search' change for
			-- last Search operation?

	identification: STRING
			-- Identifying name of element for displaying
			-- in Standard view
		do
		end

	identification_for_message: STRING
			-- Identifying name of element for displaying
			-- error and warning messages
		do
		end

	location_description: STRING
			-- Description of location of element, for use in
			-- error and warning messages
		do
		end

	size_description: STRING
			-- Description of size of element, for use in
			-- error and warning messages
		do
		end

	scribe_cut_size_description: STRING
			-- Description of cut die size of element
			-- determined by surrounding scribes, for use
			-- in error and warning messages
		do
		end

	cut_cut_size_description: STRING
			-- Description of cut die size of element determined
			-- by surrounding cuts, for use in
			-- error and warning messages
		do
		end

	offset_cut_center_description: STRING
			-- Description of offset of die center from cut
			-- die center, for use in error and warning message
		do
		end

	design_number: INTEGER
			-- Design number of element
		require
			is_project: is_project
		do
		end

	design_name: STRING
			-- Design name of element or empty string if
			-- no associated design name
		do
		end

	die_id: STRING
			-- Die ID of element or empty string if
			-- no associated die ID
		do
		end

	non_scribe_die_id: STRING
			-- Die ID of element or empty string if
			-- element is a scribe or has no associated
			-- die ID
		do
		end

	mask_site_number: INTEGER
			-- Mask site number of element
		require
			has_mask_site_number: has_mask_site_number
		do
		end

	has_mask_site_number: BOOLEAN
			-- Does element have an associated mask
			-- site number?
		do
		end

	account_number: STRING
			-- Account number of element or empty string if
			-- no associated account number
		do
		end

	account_type: STRING
			-- Account type of element or empty string if
			-- no associated account type
		do
		end

	organization_short_id: STRING
			-- Short name of organization type for element or
			-- empty string if no associated organization
		do
		end

	effective_overhead_style: STRING
			-- Effective overhead style for element
		do
		end

	effective_overhead_style_description: STRING
			-- Description of effective overhead style for
			-- element
		do
		end

	is_layout_preliminary: BOOLEAN
			-- Does element have preliminary layout (rather
			-- than final layout)?
		do
		end

	layout_status_description: STRING
			-- Description of layout status style for
			-- element
		do
		end

	required_rotation_description: STRING
			-- Description of required rotation for
			-- element
		do
			if has_required_rotation then
				Result := "Req Rot = " + required_rotation.out
				if rotation /= required_rotation then
					Result.extend ('%N')
					Result.append ("Cur Rot = " + rotation.out)
				end
			else
				Result := ""
			end
		end

	wafer_etest_name: STRING
			-- Wafer etest name for element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	alternate_wafer_etest_names: LIST [STRING]
			-- Wafer etest names for element other than
			-- the primary one
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	assigned_wafer_etest_id: STRING
			-- Wafer etest ID assigned to element or empty string
			-- if none assigned yet or if not applicable
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	unassigned_wafer_etest_ids: LIST [STRING]
			-- Wafer etest IDs not assigned to element or
			-- empty string if no unassigned etest IDs or
			-- if not applicable
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	has_wafer_etest_id: BOOLEAN
			-- Does object have associated wafer etest ID(s)?
		do
		end

	is_wafer_etest_id_assigned: BOOLEAN
			-- Does object with associated wafer etest ID(s)
			-- have a specific one assigned to it?
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	is_wafer_etest_id_missing: BOOLEAN
			-- Does object require that a wafer etest ID be
			-- assigned but not have a specific one assigned?
		do
		end

	compatible_wafer_etest_ids: LIST [STRING]
			-- Compatible wafer etest IDs for element or
			-- empty list if not applicable to this element
		require
			has_wafer_etest_id: has_wafer_etest_id
		do
		end

	all_compatible_etest_names_manufacturable: BOOLEAN
			-- Is every etest name compatible with
			-- `Current' manufacturable?
		do
		end

	any_compatible_etest_name_matches (a_partition: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Does any etest name in `compatible_wafer_etest_names'
			-- match the lot split for `a_partition'?
		do
		end

	has_constraining_wafer_etest_id: BOOLEAN
			-- Does object have constrain other elements
			-- contained within it to have a particular
			-- wafer etest ID(s)?
		do
		end

	constraining_wafer_etest_id: STRING
			-- Wafer etest ID that elements contained
			-- in `Current' are required to have if 
			-- they have a wafer etest ID
		require
			has_constraining_wafer_etest_id: has_constraining_wafer_etest_id
		do
		end

	constraining_wafer_etest_name: STRING
			-- Wafer etest ID that elements contained
			-- in `Current' are required to have if 
			-- they have a wafer etest ID
		require
			has_constraining_wafer_etest_id: has_constraining_wafer_etest_id
		do
		end

	packaging_description: STRING
			-- Description of packaging for element or 
			-- empty string if none
		do
		end

	has_ceramic_packaging: BOOLEAN
			-- Will element be packaged in at least one ceramic
			-- package?
		do
		end

	has_plastic_packaging: BOOLEAN
			-- Will element be packaged in at least one plastic
			-- package?
		do
		end

	is_commercial_project: BOOLEAN
			-- Is element a commercial project?
		do
		end

	is_mep_project: BOOLEAN
			-- Is element an MEP project?
		do
		end

	is_mep_instructional_project: BOOLEAN
			-- Is element an MEP instructional project?
		do
		end

	is_mep_research_project: BOOLEAN
			-- Is element an MEP research project?
		do
		end

	is_mep_other_project: BOOLEAN
			-- Is element an MEP project that is not
			-- instructional or research?
		do
		end

	is_other_project: BOOLEAN
			-- Is element a project whose account type is not
			-- commercial or MEP?
		do
		end

	is_die: BOOLEAN
			-- Is element a die?
		do
		end

	is_non_scribe_die: BOOLEAN
			-- Is element a die that is not a scribe
			-- (a "real" die)?
		do
		end

	is_abstract_die: BOOLEAN
			-- Is element an abstract die (die or
			-- partition)?
		do
		end

	is_cut: BOOLEAN
			-- Is element a cut?
		do
		end

	is_horizontal_cut: BOOLEAN
			-- Is element a horizontal cut line?
		do
		end

	is_vertical_cut: BOOLEAN
			-- Is element a vertical cut line?
		do
		end

	cut_position: DOUBLE
			-- Position of cut, in model coordinates (not
			-- relative to center of field)
		require
			is_cut: is_cut
		do
		end

	gridded_cut_position: DOUBLE
			-- Position of cut, in model coordinates (not
			-- relative to center of field), on grid
		require
			is_cut: is_cut
		do
			Result := grid.rounded_center (cut_position)
		end

	cut_line: RETICLE_VIEW_SUB_ELEMENT
			-- Portion of cut that is just the line
			-- (without the rectangular cut handles
			-- at each end)
		require
			is_cut: is_cut
		do
		end

	is_cut_pattern_label: BOOLEAN
			-- Is element a cut pattern label?
		do
		end

	is_field: BOOLEAN
			-- Is element a field of view?
		do
		end

	is_field_label: BOOLEAN
			-- Is element a field of view label?
		do
		end

	is_project: BOOLEAN
			-- Is element a project?
		do
		end

	is_cust_project: BOOLEAN
			-- Is element a customer project?
		do
		end

	is_unlocked_or_unassigned_project: BOOLEAN
			-- Is element a customer project that is not
			-- locked against customer update or is not assigned
			-- to this run?
		do
			Result := is_cust_project and is_unlocked_project or is_unassigned_project
		end

	is_unlocked_project: BOOLEAN
			-- Is element a customer project that is not
			-- locked against customer update?
		do
		end

	is_unassigned_project: BOOLEAN
			-- Is element a customer project that is not
			-- not assigned to this run?
		do
		end

	is_uncentered_in_cut_die_project: BOOLEAN
			-- Is element a customer project or test vehicle
			-- that is isolated in its cut die but not centered
			-- in its cut die?
		do
			Result := is_project and then not is_unisolated_in_scribe_cut_die and then is_uncentered_in_cut_die
		end

	is_test_vehicle: BOOLEAN
			-- Is element a customer project?
		do
		end

	is_partition: BOOLEAN
			-- Is element a partition?
		do
		end
	
	both_or_neither_are_partitions (other: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is it the case that either both
			-- `Current' and `other' are partitions
			-- or that neither is a partition?
		do
			Result := (is_partition and other.is_partition) or (not is_partition and not other.is_partition)
		end
	
	neither_is_partition (other: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is neither `Current' nor `other' a partition?
		do
			Result := not is_partition and not other.is_partition
		end
	
	is_scribe: BOOLEAN
			-- Is element a scribe line?
		do
		end
	
	is_horizontal_scribe: BOOLEAN
			-- Is element a horizontal scribe line?
		do
		end

	is_vertical_scribe: BOOLEAN
			-- Is element a vertical scribe line?
		do
		end

	is_default_width_scribe: BOOLEAN
			-- Is element a scribe with the default width?
		do
			Result := (is_horizontal_scribe and gridded_size_y = default_scribe_width) or (is_vertical_scribe and gridded_size_x = default_scribe_width)
		end

	is_custom_width_scribe: BOOLEAN
			-- Is element a scribe with custom (non-default) width?
		do
		end

	scribe_width: DOUBLE
			-- Width of element which is a scribe
		require
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				Result := size_y
			else
				Result := size_x
			end
		end

	default_scribe_width: DOUBLE
			-- Default width of a scribe
		do
		end

	is_non_scribe: BOOLEAN
			-- Is element a die that is not a scribe line?
		do
			Result := is_die and not is_scribe
		end
	
	is_meetable: BOOLEAN
			-- Can scribe or partition be extended to meet
			-- the edge of this element?
		do
			Result := is_scribe or is_field
		end
	
	is_movable_in_length_dimension: BOOLEAN
			-- Can scribe be extended to meet the edge of
			-- this element?
		require
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				Result := is_movable_in_x
			else
				Result := is_movable_in_y
			end
		end
	
	is_filler: BOOLEAN
			-- Is element a filler?
		do
		end

	is_vendor_geometry: BOOLEAN
			-- Is element a vendor geometry or ghost?
		do
		end

	is_movable_vendor_geometry: BOOLEAN
			-- Is element a vendor geometry/ghost
			-- that can be moved?
		do
		end

	is_selectable_by_mouse: BOOLEAN
			-- Is element selectable via mouse
			-- click or drag?
		do
			Result := is_selectable
		end

	is_resizable: BOOLEAN
			-- Can element be resized?
		do
		end

	is_below_minimum_size: BOOLEAN
			-- Is element size below the minimum allowed die size
			-- in either dimension?
		do
		end

	is_small_scribe_or_filler: BOOLEAN
			-- Is element a scribe or filler whose size is
			-- below the small object threshold in either
			-- dimension (just length for scribes)?
		do
		end

	uniform_white_space_border: DOUBLE
			-- Uniform amount of white space around element
		do
		end

	minimum_separation: DOUBLE
			-- Minimum separation between object that is a
			-- project (contains circuitry) and other objects
			-- or field of view edge
		do
		end

	needs_min_sep_white_space: BOOLEAN
			-- Does element need "minimum separation" amount of
			-- white space around it?
		do
		end

	minimum_gap: DOUBLE
			-- Gridded minimum gap needed between object and field
			-- of view edge or another object (zero gap is also
			-- allowed if not `needs_min_sep_white_space')
		do
		end

	minimum_separation_from (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Gridded minimum separation needed between `Current'
			-- and `an_element' (zero is also allowed if neither
			-- element `needs_min_sep_white_space')
		do
		end

	fields: LIST [RETICLE_VIEW_ELEMENT]
			-- List of possible fields for reticle
		do
		end

	field: RETICLE_VIEW_ELEMENT
			-- Field of view with which element is associated
		local
			l_fields: like fields
			f: RETICLE_VIEW_ELEMENT
		do
			l_fields := fields
			from
				l_fields.start
			until
				l_fields.after or Result /= Void
			loop
				f := l_fields.item
				l_fields.forth
			end
		end

	field_name: STRING
			-- Name of field of view with which element 
			-- is associated
		do
		end

	distance_from_associated_field_to_field (a_field_name: STRING): CONST_POINT
			-- Distance from center of field with which
			-- `Current' is associated to center of field
			-- named `a_field_name'
		require
			a_field_name_not_void: a_field_name /= Void
		do
		end

	is_center_in_field: BOOLEAN
			-- Is center of element inside field of view associated
			-- with element (closest field)?
		do
		end

	is_center_x_in_field: BOOLEAN
			-- Is center of element in X inside field of
			-- view associated with element (closest
			-- field)?
		do
		end

	is_center_y_in_field: BOOLEAN
			-- Is center of element in Y inside field of
			-- view associated with element (closest
			-- field)?
		do
		end

	intersects_any_field: BOOLEAN
			-- Does element bounding box intersect any
			-- field of view?
		do
		end

	intersects_more_than_one_field: BOOLEAN
			-- Does element bounding box intersect more than one
			-- field of view?
		do
		end

	intersection_with_fields: LIST [RECTANGLE]
			-- Rectangles that represent intersection of element
			-- bounding box with fields of view
		do
		end

	enclosed_by_one_field: BOOLEAN
			-- Does element bounding box completely enclosed by
			-- exactly one field of view (edges may touch field
			-- edges)?
		do
		end

	partially_outside_fields: BOOLEAN
			-- Is element bounding box partially outside fields
			-- of view (intersects at least one field but is not
			-- completely enclosed by any field)?
		do
		end

	is_too_close_to_field_edge: BOOLEAN
			-- Is at least one edge of element too close to
			-- corresponding edge of its associated field?
		do
		end

	is_edge_too_close_to_field_edge (dist: DOUBLE): BOOLEAN
			-- Is edge of element that is `dist' away from
			-- corresponding edge of its associated field
			-- too close?
		do
		end

	distance_in_x_from (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Gridded distance of element from
			-- `an_element' in X dimension.  Result is
			-- zero if elements overlap in X
		do
		end

	distance_in_y_from (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Gridded distance of element from
			-- `an_element' in Y dimension.  Result is
			-- zero if elements overlap in Y
		do
		end

	abuts_field_edge_on_length_side: BOOLEAN
			-- Does `Current' abut edge of its associated
			-- field on either of its length sides?
		require
			is_scribe: is_scribe
		do
			Result := is_horizontal_scribe and (abuts_bottom_field_edge or abuts_top_field_edge) or
				is_vertical_scribe and (abuts_left_field_edge or abuts_right_field_edge)
		end

	abuts_left_field_edge: BOOLEAN
			-- Does left side of `Current' abut the left edge 
			-- of its associated field?
		do
		end

	abuts_right_field_edge: BOOLEAN
			-- Does right side of `Current' abut the right edge 
			-- of its associated field?
		do
		end

	abuts_bottom_field_edge: BOOLEAN
			-- Does bottom side of `Current' abut the bottom edge 
			-- of its associated field?
		do
		end

	abuts_top_field_edge: BOOLEAN
			-- Does top side of `Current' abut the top edge 
			-- of its associated field?
		do
		end

	left_to_field_edge: DOUBLE
			-- Gridded distance from left edge of element to left
			-- edge of its associated field (positive if
			-- element edge is to right of field edge)
		do
		end

	right_to_field_edge: DOUBLE
			-- Gridded distance from right edge of element to right
			-- edge of its associated field (positive if
			-- element edge is to left of field edge)
		do
		end

	top_to_field_edge: DOUBLE
			-- Gridded distance from top edge of element to top
			-- edge of its associated field (positive if
			-- element edge is below field edge)
		do
		end

	bottom_to_field_edge: DOUBLE
			-- Gridded distance from bottom edge of
			-- element to bottom edge of its associated
			-- field (positive if element edge is above
			-- field edge)
		do
		end

	abuts_element_on_both_sides (list: LIST [RETICLE_VIEW_ELEMENT]): BOOLEAN
			-- Do both length sides of `Current' abut 
			-- edge of one of the elements of `list'?
		require
			is_scribe: is_scribe
		do
			Result := is_horizontal_scribe and abuts_bottom_element_edge (list) and abuts_top_element_edge (list) or
				is_vertical_scribe and abuts_left_element_edge (list) and abuts_right_element_edge (list)
		end

	abuts_bottom_element_edge (list: LIST [RETICLE_VIEW_ELEMENT]): BOOLEAN
			-- Does top edge of `Current' abut 
			-- bottom edge of one of the elements of `list'?
		local
			pos, low, high: DOUBLE
			elem_pos, elem_low, elem_high: DOUBLE
			elem: RETICLE_VIEW_ELEMENT
		do
			pos := grid.rounded_edge (top_y)
			low := grid.rounded_edge (left_x)
			high := grid.rounded_edge (right_x)
			from
				list.start
			until
				list.after or Result
			loop
				elem := list.item
				elem_pos := grid.rounded_edge (elem.bottom_y)
				elem_low := grid.rounded_edge (elem.left_x)
				elem_high := grid.rounded_edge (elem.right_x)
				Result := Result or pos = elem_pos and low < elem_high and high > elem_low
				list.forth
			end
		end

	abuts_top_element_edge (list: LIST [RETICLE_VIEW_ELEMENT]): BOOLEAN
			-- Does bottom edge of `Current' abut 
			-- top edge of one of the elements of `list'?
		local
			pos, low, high: DOUBLE
			elem_pos, elem_low, elem_high: DOUBLE
			elem: RETICLE_VIEW_ELEMENT
		do
			pos := grid.rounded_edge (bottom_y)
			low := grid.rounded_edge (left_x)
			high := grid.rounded_edge (right_x)
			from
				list.start
			until
				list.after or Result
			loop
				elem := list.item
				elem_pos := grid.rounded_edge (elem.top_y)
				elem_low := grid.rounded_edge (elem.left_x)
				elem_high := grid.rounded_edge (elem.right_x)
				Result := Result or pos = elem_pos and low < elem_high and high > elem_low
				list.forth
			end
		end

	abuts_left_element_edge (list: LIST [RETICLE_VIEW_ELEMENT]): BOOLEAN
			-- Does right edge of `Current' abut 
			-- left edge of one of the elements of `list'?
		local
			pos, low, high: DOUBLE
			elem_pos, elem_low, elem_high: DOUBLE
			elem: RETICLE_VIEW_ELEMENT
		do
			pos := grid.rounded_edge (right_x)
			low := grid.rounded_edge (bottom_y)
			high := grid.rounded_edge (top_y)
			from
				list.start
			until
				list.after or Result
			loop
				elem := list.item
				elem_pos := grid.rounded_edge (elem.left_x)
				elem_low := grid.rounded_edge (elem.bottom_y)
				elem_high := grid.rounded_edge (elem.top_y)
				Result := Result or pos = elem_pos and low < elem_high and high > elem_low
				list.forth
			end
		end

	abuts_right_element_edge (list: LIST [RETICLE_VIEW_ELEMENT]): BOOLEAN
			-- Does left edge of `Current' abut 
			-- right edge of one of the elements of `list'?
		local
			pos, low, high: DOUBLE
			elem_pos, elem_low, elem_high: DOUBLE
			elem: RETICLE_VIEW_ELEMENT
		do
			pos := grid.rounded_edge (left_x)
			low := grid.rounded_edge (bottom_y)
			high := grid.rounded_edge (top_y)
			from
				list.start
			until
				list.after or Result
			loop
				elem := list.item
				elem_pos := grid.rounded_edge (elem.right_x)
				elem_low := grid.rounded_edge (elem.bottom_y)
				elem_high := grid.rounded_edge (elem.top_y)
				Result := Result or pos = elem_pos and low < elem_high and high > elem_low
				list.forth
			end
		end

	x_outside_all_fields (a_value: DOUBLE): BOOLEAN
			-- Is `a_value' outside all fields of view in X
			-- dimension or on field edge?
		do
		end

	y_outside_all_fields (a_value: DOUBLE): BOOLEAN
			-- Is `a_value' outside all fields of view in Y
			-- dimension or on field edge?
		do
		end

	closest_field_edge_above (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field lower edge which is
			-- closest to and above or equal to `a_value'
		do
		end

	closest_field_edge_below (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field upper edge which is closest to
			-- and below or equal to `a_value'
		do
		end

	closest_field_edge_to_left (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field right edge which is
			-- closest to and left of or equal to
			-- `a_value'
		do
		end

	closest_field_edge_to_right (a_value: DOUBLE): DOUBLE
			-- Y coordinate of field left edge which is
			-- closest to and right of or equal to
			-- `a_value'
		do
		end

	intersects (an_element: like Current): BOOLEAN
			-- Is intersection of `an_element' and `Current'
			-- non-empty?
		do
		end

	encloses (an_element: like Current): BOOLEAN
			-- Does model bounding box of `Current' enclose
			-- that of `an_element'?
		do
		end

	model_bounding_box: CONST_RECTANGLE
			-- Bounding box of element in model (not in view)
		do
		end

	size_x: DOUBLE
			-- Size of associated model element in X dimension
		do
		end

	size_y: DOUBLE
			-- Size of associated model element in X dimension
		do
		end

	has_zero_size_in_x: BOOLEAN
			-- Does `Current' actually have zero size in X
			-- dimension?
		do
		end
		
	has_zero_size_in_y: BOOLEAN
			-- Does `Current' actually have zero size in Y
			-- dimension?
		do
		end
		
	has_size_x: BOOLEAN = True
			-- Does `Current' have an associated size in X
			-- dimension?

	has_size_y: BOOLEAN = True
			-- Does `Current' have an associated size in Y
			-- dimension?

	size_x_string: STRING
			-- Size of associated model element in X dimension
			-- as a string
		do
			Result := grid.formatted_number (size_x)
		end

	size_y_string: STRING
			-- Size of associated model element in Y dimension
			-- as a string
		do
			Result := grid.formatted_number (size_y)
		end

	gridded_size_x: DOUBLE
			-- Size of associated model element in X dimension,
			-- rounded to grid
		do
			Result := grid.rounded_size (size_x)
		end

	gridded_size_y: DOUBLE
			-- Size of associated model element in X dimension,
			-- rounded to grid
		do
			Result := grid.rounded_size (size_y)
		end

	slop_x: DOUBLE
			-- Difference between maximum die size and minimum
			-- die size in X dimension
		require
			has_max_die_size: has_max_die_size
		do
			Result := max_die_size.x - size_x
		end

	slop_y: DOUBLE
			-- Difference between maximum die size and minimum
			-- die size in X dimension
		require
			has_max_die_size: has_max_die_size
		do
			Result := max_die_size.y - size_y
		end

	left_x: DOUBLE
			-- Left edge of minimum bounding box of associated
			-- model element in X dimension
		do
		end

	right_x: DOUBLE
			-- Right edge of minimum bounding box of associated
			-- model element in X dimension
		do
		end

	top_y: DOUBLE
			-- Top edge of minimum bounding box of associated
			-- model element in Y dimension
		do
		end

	bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box of associated
			-- model element in X dimension
		do
		end

	gridded_left_x: DOUBLE
			-- Left edge of minimum bounding box of associated
			-- model element in X dimension, placed on grid
		do
			Result := grid.rounded_edge (left_x)
		end

	gridded_right_x: DOUBLE
			-- Right edge of minimum bounding box of associated
			-- model element in X dimension, placed on grid
		do
			Result := grid.rounded_edge (right_x)
		end

	gridded_top_y: DOUBLE
			-- Top edge of minimum bounding box of associated
			-- model element in Y dimension, placed on grid
		do
			Result := grid.rounded_edge (top_y)
		end

	gridded_bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box of associated
			-- model element in X dimension, placed on grid
		do
			Result := grid.rounded_edge (bottom_y)
		end

	center_x: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in X dimension
		do
		end

	center_y: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in Y dimension
		do
		end

	field_relative_center_x: DOUBLE
			-- Center of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
		end

	field_relative_center_y: DOUBLE
			-- Center of minimum bounding box in Y dimension,
			-- relative to center of field with which element
			-- is associated
		do
		end

	field_relative_left_x: DOUBLE
			-- Left edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
		end

	field_relative_right_x: DOUBLE
			-- Right edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
		end

	field_relative_top_y: DOUBLE
			-- Top edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
		end

	field_relative_bottom_y: DOUBLE
			-- Bottom edge of minimum bounding box in X dimension,
			-- relative to center of field with which element
			-- is associated
		do
		end

	gridded_center_x: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in X dimension, placed on grid
		do
			Result := grid.rounded_center (center_x)
		end

	gridded_center_y: DOUBLE
			-- Center of minimum bounding box of associated
			-- model element in Y dimension, placed on grid
		do
			Result := grid.rounded_center (center_y)
		end

	has_gridded_center_x: BOOLEAN = True
			-- Does `Current' have an associated gridded
			-- center in X dimension?

	has_gridded_center_y: BOOLEAN = True
			-- Does `Current' have an associated gridded center
			-- in Y dimension?

	gridded_center_x_string: STRING
			-- Gridded center of associated model element
			-- in X dimension as a string
		do
			Result := grid.formatted_number (gridded_center_x)
		end

	gridded_center_y_string: STRING
			-- Gridded center of associated model element
			-- in Y dimension as a string
		do
			Result := grid.formatted_number (gridded_center_y)
		end

	has_fixed_in_x: BOOLEAN = True
			-- Does `Current' have an associated "fixed in X"
			-- attribute?

	has_fixed_in_y: BOOLEAN = True
			-- Does `Current' have an associated "fixed in Y"
			-- attribute?

	has_max_die_size: BOOLEAN
			-- Does `Current' have a maximum die size?
		do
			Result := max_die_size /= Void
		end

	max_die_size: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes and user-specified limits.
			-- Void indicates no maximum
		require
			is_die: is_die
		do
		end

	packaging_max_die_size: CONST_POINT
			-- Maximum die size, as constrained by package
			-- cavity sizes.  Void indicates no maximum
		require
			is_die: is_die
		do
		end

	user_max_die_size: CONST_POINT
			-- Maximum die size, as constrained by 
			-- user-specified limits.  Void indicates no maximum
		require
			is_die: is_die
		do
		end

	is_oversize_die_always_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed unconditionally?
		require
			is_die: is_die
		do
		end

	is_oversize_die_possibly_allowed: BOOLEAN
			-- Is a die that is bigger than `max_die_size'
			-- allowed but only under specific conditions?
		require
			is_die: is_die
		do
		end

	oversize_die_inclusion_short_desc: STRING
			-- Short description of whether a
			-- die that is bigger than `max_die_size'
			-- can be included on the run
		require
			is_die: is_die
		do
			if is_oversize_die_always_allowed then
				Result := "Opt"
			elseif is_oversize_die_possibly_allowed then
				Result := "Cond"
			else
				Result := "Mand"
			end
		end

	oversize_die_inclusion_conditions: STRING
			-- Description of conditions under which 
			-- die that is bigger than `max_die_size'
			-- allowed
		require
			is_die: is_die
			oversize_die_possibly_allowed: is_oversize_die_possibly_allowed
		do
		end

	has_sub_elements: BOOLEAN
			-- Is element composed of more than one sub-element,
			-- each of which has its own size and center?
		do
		end

	sub_elements: LIST [like anchored_view_sub_element]
			-- Sub-elements of `Current'

	packing_name: STRING
			-- Packing name for element
		do
		end

	has_packing_name: BOOLEAN = True
			-- Does `Current' have an associated packing name?

	all_copies: RETICLE_VIEW_ELEMENT_LIST
			-- All copies of this element

	all_copies_selected: BOOLEAN
			-- Are all copies of element currently selected?
		do
			Result := all_copies.all_elements_selected
		end

	any_copies_selected: BOOLEAN
			-- Are any copies of element currently selected?
		do
			Result := all_copies.any_elements_selected
		end

	is_partially_selected: BOOLEAN
			-- Is element partially selected (some but not all
			-- copies selected)?
		do
			Result := any_copies_selected and not all_copies_selected
		end

	any_copies_stationary_orientation: BOOLEAN
			-- Do any copies of element have a stationary
			-- orientation, which cannot be changed
		do
			Result := all_copies.any_elements_stationary_orientation
		end

	any_copies_rotation_locked: BOOLEAN
			-- Are any copies of element locked to prevent
			-- changed of orientation?
		do
			Result := all_copies.any_elements_rotation_locked
		end

	is_processed: BOOLEAN
			-- Have any copies of element been processed?
		do
			Result := all_copies.is_processed
		end

	rotation_move_distance: DOUBLE
			-- Distance `Current' had to be moved in both
			-- X and Y after rotation to keep the edges on
			-- grid
		do
			Result := all_copies.rotation_move_distance
		end

	participates_in_measure: BOOLEAN
			-- Does `Current' participate in measure in X/Y
			-- operations?
		do
		end

	participates_in_scan: BOOLEAN
			-- Does `Current' participate in horizontal
			-- and vertical scan operations?
		do
		end

	is_misaligned_with (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is element misaligned with `an_element'?
		require
			is_scribe: is_scribe
			an_element_is_scribe: an_element.is_scribe
			same_orientation_if_horizontal: is_horizontal_scribe implies an_element.is_horizontal_scribe
			same_orientation_if_vertical: is_vertical_scribe implies an_element.is_vertical_scribe
		local
			low, high, other_low, other_high: DOUBLE
		do
			if is_horizontal_scribe then
				low := gridded_bottom_y
				high := gridded_top_y
				other_low := an_element.gridded_bottom_y
				other_high := an_element.gridded_top_y
			else
				low := gridded_left_x
				high := gridded_right_x
				other_low := an_element.gridded_left_x
				other_high := an_element.gridded_right_x
			end
			Result := low <= other_high and high >= other_low and (low /= other_low or high /= other_high)
		end

	misalignment_amount (an_element: RETICLE_VIEW_ELEMENT): DOUBLE
			-- Amount by which `an_element' is misaligned with
			-- `Current'
		require
			is_scribe: is_scribe
			an_element_is_scribe: an_element.is_scribe
			same_orientation_if_horizontal: is_horizontal_scribe implies an_element.is_horizontal_scribe
			same_orientation_if_vertical: is_vertical_scribe implies an_element.is_vertical_scribe
		local
			low, high, other_low, other_high: DOUBLE
			low_diff, high_diff: DOUBLE
		do
			if is_horizontal_scribe then
				low := gridded_bottom_y
				high := gridded_top_y
				other_low := an_element.gridded_bottom_y
				other_high := an_element.gridded_top_y
			else
				low := gridded_left_x
				high := gridded_right_x
				other_low := an_element.gridded_left_x
				other_high := an_element.gridded_right_x
			end
			low_diff := grid.rounded_edge ((low - other_low).abs)
			high_diff := grid.rounded_edge ((high - other_high).abs)
			if low_diff = 0.0 then
				Result := high_diff
			elseif high_diff = 0.0 then
				Result := low_diff
			else
				Result := low_diff.min (high_diff)
			end
		end

	is_marked: BOOLEAN
			-- Has element been marked as being special in some
			-- way?  Meaning of marked depends on caller

	invalid_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of invalid elements, with which `Current'
			-- has some kind of validity problem, such as
			-- overlap.  List is used to avoid issuing duplicate
			-- error messages for objects that fall into more
			-- than one area and hence may be compared more than
			-- once with `Current' for validity problems

	scribe_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- centers of surrounding scribes for edges of cut die
		do
		end

	oriented_scribe_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using centers of
			-- surrounding scribes for edges of cut die
		do
		end

	scribe_cut_die_size_x: DOUBLE
			-- Size of cut die in X dimension
		do
		end

	scribe_cut_die_size_y: DOUBLE
			-- Size of cut die in Y dimension
		do
		end

	edge_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- closest edges of surrounding scribes for edges
			-- of cut die
		do
		end

	oriented_cut_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project,
			-- oriented the same as the die (i.e., rotated
			-- if die is rotated), using cuts for edges
			-- of cut die
		do
		end

	cut_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- cuts for edges of cut die
		do
		end

	cut_or_scribe_cut_die: CONST_RECTANGLE
			-- Last computed cut die for a project, using
			-- `cut_cut_die' if not Void and otherwise
			-- `scribe_cut_die'
		do
			Result := cut_cut_die
			if Result = Void then
				Result := scribe_cut_die
			end
		end

	cut_cut_die_size_x: DOUBLE
			-- Size of cut die in X dimension, using centers
			-- of surrounding cuts for edges of cut die
		do
		end

	cut_cut_die_size_y: DOUBLE
			-- Size of cut die in Y dimension, using centers
			-- of surrounding cut for edges of cut die
		do
		end

	has_surrounding_cuts: BOOLEAN
			-- Were there cuts on all four sides of project on
			-- last computation of cut die

	isolated_cut_die: CONST_RECTANGLE
			-- Cut die for an isolated project, determined
			-- from its cut die site
		do
		end

	is_in_current_cut_pattern_lot: BOOLEAN
			-- Does `Current' have the same lot (same 
			-- wafer_etest_name) as the lot for the current cut
			-- pattern?
		local
			cp_etest_name: STRING
			alt_etest_names: LIST [STRING]
		do
			if has_wafer_etest_id then
				alt_etest_names := alternate_wafer_etest_names
				Result := cp_etest_name.is_equal (wafer_etest_name) or (alt_etest_names /= Void and then alt_etest_names.has (cp_etest_name))
			else
				Result := True
			end
		end

	is_floating_scribe: BOOLEAN
			-- Is element a floating scribe?  Only valid
			-- after being computed for certain validity
			-- check operations

	is_misaligned_scribe: BOOLEAN
			-- Is element a scribe misaligned with another
			-- scribe of the same orientation (both must
			-- intersect at least one field of view)?  Value
			-- is the one computed for last validity check
			-- operation that checked for misaligned scribes

	is_empty_partition: BOOLEAN
			-- Is element a partition that is empty
			-- (contains no cust projects or test
			-- vehicles)?  Value is the one computed for
			-- last validity check operation that checked
			-- for empty partitions

	is_partition_with_invalid_edge: BOOLEAN
			-- Is element a partition with an edge in an
			-- invalid position (not on field edge or on
			-- near edge of scribe)?  Value is the one
			-- computed for last validity check operation
			-- that checked for partitions with an invalid
			-- edge

	is_not_contained_in_partition: BOOLEAN
			-- Is element a project that is not fully
			-- contained in any partition?  Value is the
			-- one computed for last validity check
			-- operation that checked for non-contained
			-- projects

	is_partially_in_partition: BOOLEAN
			-- Is element a scribe that intersects a
			-- partition but is not fully contained in one?  Value is the
			-- one computed for last validity check
			-- operation that checked for "partially 
			-- in partition" scribes

	is_invalidly_placed_scribe: BOOLEAN
			-- Is element a scribe that is outside all
			-- partitions and abuts field edge on its
			-- length side, when there are partitions?
			-- Value is the one computed for last validity
			-- check operation that checked for invalidly
			-- placed scribes

	has_wrong_lot_for_partition: BOOLEAN
			-- Is element a project that has the wrong lot
			-- (different than the one for its containing
			-- partition)?  Value is the one computed for
			-- last validity check operation that checked
			-- for "wrong lot" projects

	is_cut_too_close_to_adjacent_cut: BOOLEAN
			-- Is element a cut that is too close to an
			-- adjacent cut of the same orientation?
			-- Value is the one computed for
			-- last validity check operation that checked
			-- for "too close to adjacent cut" cuts

	is_cut_too_close_to_adjacent_strip_cut: BOOLEAN
			-- Is element a cut that is too close to an
			-- adjacent cut of the same orientation in a
			-- cut pattern that produces strips, resulting
			-- in a wafer strip that whose width is below
			-- the minimum?  Value is the one computed for
			-- last validity check operation that checked
			-- for "too close to adjacent strip cut" cuts

	all_partitions: LIST [RETICLE_VIEW_ELEMENT]
			-- List of current partitions
		do
		end

	last_duplicated_element: RETICLE_VIEW_ELEMENT
			-- Last new element create by call to `duplicate'

	center_fits_in_database (a_center: CONST_POINT): BOOLEAN
			-- Will model location `a_center' fit in database
			-- (in PACKING_DIE_LOCATIONS table)?  `a_center'
			-- is a location in model coordinates, not a
			-- field-relative center
		require
			a_center_not_void: a_center /= Void
		do
			Result := view.center_fits_in_database (a_center)
		end

	moved_center_fits_in_database (a_distance: CONST_POINT): BOOLEAN
			-- Does model location moved by `a_distance' fit 
			-- in database?
		require
			a_distance_not_void: a_distance /= Void
		do
		end

	intersecting_mbb_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this element's bounding box
			-- intersects.  Used
			-- to speed up validity checking

	intersecting_scribe_cut_die_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this element's cut die
			-- (determined by surrounding scribes)
			-- intersects.  Used to speed up validity
			-- checking

	intersecting_cut_cut_die_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this element's cut die
			-- (determined by surrounding cuts)
			-- intersects.  Used to speed up validity
			-- checking

	intersecting_hscribe_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this horizontal scribe's
			-- bounding box intersects.  Used to speed up
			-- validity checking

	intersecting_vscribe_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this vertical scribe's
			-- bounding box intersects.  Used to speed up
			-- validity checking

	intersecting_scribe_areas: LIST [RETICLE_MODEL_AREA]
			-- Areas of model this scribe's bounding box
			-- intersects
		require
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				Result := intersecting_hscribe_areas
			else
				Result := intersecting_vscribe_areas
			end
		end

	is_overlapping_another_object: BOOLEAN
			-- Does element overlap another element (both must
			-- intersect at least one field of view)?  Value
			-- is the one computed for last validity check
			-- operation that checked for overlapping objects

	is_too_close_to_another_object: BOOLEAN
			-- Is element too close to another element but
			-- not overlapping it (both must intersect at
			-- least one field of view)?  Value is the one
			-- computed for last validity check operation
			-- that checked for "too close" objects

	is_unisolated_in_scribe_cut_die: BOOLEAN
			-- Is element not isolated in its cut die determined
			-- by surrounding scribes?
			-- Value is the one computed for last validity
			-- check operation that checked for unisolated
			-- projects

	is_unisolated_in_cut_cut_die: BOOLEAN
			-- Is element not isolated in its cut die determined
			-- by surrounding cuts?
			-- Value is the one computed for last validity
			-- check operation that checked for unisolated
			-- projects

	is_isolating_cut: BOOLEAN
			-- Is element a cut that helps isolate at least one
			-- isolate site?

	misaligned_with_other_scribe: BOOLEAN
			-- Is element which is a scribe misaligned with
			-- another scribe of the same orientation?
		require
			is_scribe: is_scribe
		local
			areas: LIST [RETICLE_MODEL_AREA]
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_scribe_areas
				from
					areas.start
				until
					areas.after or Result
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after or Result
					loop
						elem := elems.item
						if elem /= Current and is_misaligned_with (elem) then
							Result := True
						end
						elems.forth
					end
					areas.forth
				end
			end
		end

	contains_project: BOOLEAN
			-- Is element a partition that contains
			-- at least one project, as determined
			-- by project center?
		require
			is_partition: is_partition
		local
			mbb: CONST_RECTANGLE
			areas: LIST [RETICLE_MODEL_AREA]
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			areas := intersecting_mbb_areas
			from
				areas.start
			until
				areas.after or Result
			loop
				area := areas.item
				elems := area.intersecting_elements
				from
					elems.start
				until
					elems.after or Result
				loop
					elem := elems.item
					if elem.is_project and mbb.truly_contains (create {CONST_POINT}.make_xy (elem.center_x, elem.center_y)) then
						Result := True
					end
					elems.forth
				end
				areas.forth
			end
		end

	is_contained_in_any_element (list: LIST [RETICLE_VIEW_ELEMENT]): BOOLEAN
			-- Is `Current' mbb fully contained in at
			-- least one element of `list'?
		local
			elem: RETICLE_VIEW_ELEMENT
		do
			from
				list.start
			until
				list.after or Result
			loop
				elem := list.item
				list.forth
			end
		end

	intersects_any_element (list: LIST [RETICLE_VIEW_ELEMENT]): BOOLEAN
			-- Does mbb of `Current' intersect the mbb of
			-- any element of `list'?
		local
			elem: RETICLE_VIEW_ELEMENT
		do
			from
				list.start
			until
				list.after or Result
			loop
				elem := list.item
				list.forth
			end
		end

	unique_containing_element (list: LIST [RETICLE_VIEW_ELEMENT]): RETICLE_VIEW_ELEMENT
			-- Unique element of `list' that contains
			-- center of `Current' or Void if none or
			-- more than one such element
		local
			p: CONST_POINT
			done: BOOLEAN
			elem: RETICLE_VIEW_ELEMENT
		do
			create p.make_xy (center_x, center_y)
			from
				list.start
			until
				list.after or done
			loop
				elem := list.item
				list.forth
			end
		end

	mismatches_partition_lot (a_partition: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is `Current' assigned to a lot that is
			-- different than the constraining lot for
			-- `a_partition'?
		local
			lot, required_lot: like assigned_wafer_etest_id
		do
			lot := assigned_wafer_etest_id
			required_lot := a_partition.constraining_wafer_etest_id
			Result := not lot.is_empty and not lot.is_equal (required_lot)
		end

	mismatches_containing_partition_lot (list: LIST [RETICLE_VIEW_ELEMENT]): BOOLEAN
			-- Is there a unique partition in `list' that
			-- contains the center of `Current' and is
			-- `Current' also assigned to a lot that is
			-- different than the constraining lot for
			-- that unique containing partition?
		local
			l_partition: RETICLE_VIEW_ELEMENT
		do
			l_partition := unique_containing_element (list)
			if l_partition /= Void then
				Result := mismatches_partition_lot (l_partition)
			end
		end

	overlaps_other_object: BOOLEAN
			-- Does element overlap another object?  Considers
			-- only dies
		local
			areas: LIST [RETICLE_MODEL_AREA]
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_mbb_areas
				from
					areas.start
				until
					areas.after or Result
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after or Result
					loop
						elem := elems.item
						elems.forth
					end
					areas.forth
				end
			end
		end

	is_too_close_to_object: BOOLEAN
			-- Is element too close to another object?  Considers
			-- only dies
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			bigger_mbb, other_mbb: RECTANGLE
			gap, other_grow, dist_x, dist_y: DOUBLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_mbb_areas
				gap := minimum_gap
				if gap > 0.0 then
					create bigger_mbb.make_copy (mbb)
					bigger_mbb.size_by (2.0 * gap, 2.0 * gap)
					from
						areas.start
					until
						areas.after or Result
					loop
						area := areas.item
						elems := area.intersecting_elements
						from
							elems.start
						until
							elems.after or Result
						loop
							elem := elems.item
							if elem /= Current and then neither_is_partition (elem) and then not mbb.intersects (other_mbb) then
								other_grow := -2.0 * elem.uniform_white_space_border
								other_mbb.size_by (other_grow, other_grow)
								dist_x := distance_in_x_from (elem)
								dist_y := distance_in_y_from (elem)
								Result := bigger_mbb.intersects (other_mbb) and (needs_min_sep_white_space or elem.needs_min_sep_white_space or dist_x /= 0.0 or dist_y /= 0.0)
							end
							elems.forth
						end
						areas.forth
					end
				end
			end
		end

	is_scribe_cut_die_too_big_for (a_max_die_size: CONST_POINT): BOOLEAN
			-- Does die size of element determined by surrounding
			-- scribes exceed `a_max_die_size'?
		local
			l_max: CONST_POINT
			sx, sy: DOUBLE
		do
			if is_die then
				l_max := a_max_die_size
				if l_max /= Void and scribe_cut_die /= Void then
					sx := grid.rounded_size (scribe_cut_die_size_x)
					sy := grid.rounded_size (scribe_cut_die_size_y)
					Result := sx > l_max.x or sy > l_max.y
				end
			end
		end

	is_scribe_cut_die_too_big: BOOLEAN
			-- Does die size of element determined by surrounding
			-- scribes exceed its maximum size?
		do
			if is_die then
				Result := is_scribe_cut_die_too_big_for (max_die_size)
			end
		end

	is_cut_cut_die_too_big_for (a_max_die_size: CONST_POINT): BOOLEAN
			-- Does die size of element determined by surrounding
			-- cuts exceed `a_max_die_size'?
		local
			l_max: CONST_POINT
			sx, sy: DOUBLE
		do
				l_max := a_max_die_size
				if l_max /= Void and cut_cut_die /= Void then
					sx := grid.rounded_center (cut_cut_die_size_x)
					sy := grid.rounded_center (cut_cut_die_size_y)
					Result := sx > l_max.x or sy > l_max.y
				end
		end

	is_cut_cut_die_too_big: BOOLEAN
			-- Does die size of element determined by surrounding
			-- cuts exceed its maximum size?
		do
				Result := is_cut_cut_die_too_big_for (max_die_size)
		end

	is_cut_cut_die_invalid: BOOLEAN
			-- Is die size of element determined by surrounding
			-- cuts invalid?
		do
		end

	has_wrong_lot_for_cut_pattern: BOOLEAN
			-- Is element one that is isolated by cut pattern but 
			-- has an assigned lot that is different from the
			-- one for the cut pattern?
		do
		end

	has_non_default_cut_die_size: BOOLEAN
			-- Is element one that is isolated by cut pattern and
			-- has a cut die size determined by surrounding cuts
			-- that is different from the cut die size determined
			-- by surrounding scribes?
		local
			l_cut_cut_die: like cut_cut_die
		do
			l_cut_cut_die := cut_cut_die
		end

	shares_cut_die_with_others: BOOLEAN
			-- Is element one that is isolated by cut pattern and
			-- shares its cut die determined by surrounding cuts
			-- either partially or fully with another non-scribe
			-- die?
		do
			Result := is_unisolated_in_cut_cut_die
		end

	is_extra_cut: BOOLEAN
			-- Is element an extra cut that does not help isolate
			-- any isolated sites?
		do
		end

	offset_cut_center_x: DOUBLE
			-- Offset of center of die from center of cut die
			-- in X dimension, using closest edges of surrounding
			-- scribes instead of centers
		require
			scribe_cut_die_not_void: scribe_cut_die /= Void
		do
		end

	offset_cut_center_y: DOUBLE
			-- Offset of center of die from center of cut die
			-- in X dimension, using closest edges of surrounding
			-- scribes instead of centers
		require
			scribe_cut_die_not_void: scribe_cut_die /= Void
		do
		end

	is_uncentered_in_cut_die: BOOLEAN
			-- Is element not centered in its cut die (using
			-- closest edges of surrounding scribes, instead
			-- of centers)?
		local
			l_cut_die: like edge_cut_die
			diff_x, diff_y: DOUBLE
		do
			l_cut_die := edge_cut_die
			if is_project and l_cut_die /= Void then
				diff_x := (grid.rounded_edge (left_x - l_cut_die.low_x) - grid.rounded_edge (l_cut_die.high_x - right_x)).abs
				diff_x := grid.rounded_edge (diff_x)
				diff_y := (grid.rounded_edge (bottom_y - l_cut_die.low_y) - grid.rounded_edge (l_cut_die.high_y - top_y)).abs
				diff_y := grid.rounded_edge (diff_y)
				Result := diff_x > grid.edge_resolution or diff_y > grid.edge_resolution
			end
		end

	unisolated_in_scribe_cut_die: BOOLEAN
			-- Is element not isolated in its cut die, determined
			-- by surrounding scribes?
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if is_project and intersects_any_field then
				areas := intersecting_scribe_cut_die_areas
				mbb := scribe_cut_die
				from
					areas.start
				until
					areas.after or Result
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after or Result
					loop
						elem := elems.item
						if elem /= Current and then mbb.intersects (elem.scribe_cut_die) then
							Result := True
						end
						elems.forth
					end
					areas.forth
				end
			end
		end

	unisolated_in_cut_cut_die: BOOLEAN
			-- Does element share its cut die, determined
			-- by surrounding cuts, with other non-scribe
			-- dies?
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if can_be_isolated then
				areas := intersecting_cut_cut_die_areas
				mbb := cut_cut_die
				if mbb /= Void then
					from
						areas.start
					until
						areas.after or Result
					loop
						area := areas.item
						elems := area.intersecting_elements
						from
							elems.start
						until
							elems.after or Result
						loop
							elem := elems.item
							if elem /= Current and then mbb.intersects (elem.scribe_cut_die) then
								Result := True
							end
							elems.forth
						end
						areas.forth
					end
				end
			end
		end

	can_be_isolated: BOOLEAN
			-- Is element allowed to be isolated, based on its
			-- type?
		do
		end

	appears_isolated: BOOLEAN
			-- Does element appear to be isolated in current 
			-- cut pattern (is scribe cut die same as cut cut die)?
		local
			l_cut_cut_die: like cut_cut_die
		do
			l_cut_cut_die := cut_cut_die
			Result := l_cut_cut_die /= Void and then l_cut_cut_die.is_same_box (scribe_cut_die)
		end

	is_reworkable: BOOLEAN
			-- Is element currently reworkable?
		do
		end

	is_hidden: BOOLEAN
			-- Is element currently hidden (not displayed
			-- in most views)?

	is_displayed: BOOLEAN
			-- Is element currently displayed
			-- (either element is not hidden or 
			-- current view always displayed hidden elements)?
		do
		end

	has_required_rotation: BOOLEAN
			-- Must object have specific rotation?
		do
		end

	required_rotation: INTEGER
			-- Rotation object is required to have in degrees clockwise
		require
			has_required_rotation: has_required_rotation
		do
		end

	has_wrong_rotation: BOOLEAN
			-- Does object's current rotation not match
			-- its required rotation?
		do
			Result := has_required_rotation and then rotation /= required_rotation
		end

feature -- Modification

	set_is_hidden_element_displayed (a_value: like is_hidden_element_displayed)
			-- Set `is_hidden_element_displayed' to `a_value'.  Set
			-- `is_hidden_element_displayed_changed' to indicate whether
			-- the value changed
		do
			is_hidden_element_displayed_changed := is_hidden_element_displayed /= a_value
			is_hidden_element_displayed := a_value
		ensure
			value_set: is_hidden_element_displayed = a_value
		end

	set_is_selected (b: BOOLEAN)
			-- Set `is_selected' to `b'.  Set `is_selected_changed'
			-- to indicate whether the value of `is_selected'
			-- was changed
		require
			is_selectable: is_selectable
		do
			is_selected_changed := is_selected /= b
			is_selected := b
		ensure
			value_set: is_selected = b
		end

	set_included_in_changed_region (b: BOOLEAN)
			-- Set `included_in_changed_region' to `b'
		do
			included_in_changed_region := b
		ensure
			value_set: included_in_changed_region = b
		end

	set_matched_last_search (b: BOOLEAN)
			-- Set `matched_last_search' to `b'
		do
			matched_last_search_changed := matched_last_search /= b
			matched_last_search := b
		ensure
			value_set: matched_last_search = b
		end

	set_all_copies (a_list: like all_copies)
			-- Set `all_copies' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			all_copies := a_list
		ensure
			all_copies_set: all_copies = a_list
		end

	remove_from_all_copies
			-- Remove `Current' from `all_copies'
		do
			all_copies.remove (Current)
		end

	set_is_overlapping_another_object (b: BOOLEAN)
			-- Set `is_overlapping_another_object' to `b'
		do
			is_overlapping_another_object := b
		end

	set_is_too_close_to_another_object (b: BOOLEAN)
			-- Set `is_too_close_to_another_object' to `b'
		do
			is_too_close_to_another_object := b
		end

	set_is_unisolated_in_scribe_cut_die (b: BOOLEAN)
			-- Set `is_unisolated_in_scribe_cut_die' to `b'
		do
			is_unisolated_in_scribe_cut_die := b
		end

	set_is_unisolated_in_cut_cut_die (b: BOOLEAN)
			-- Set `is_unisolated_in_cut_cut_die' to `b'
		do
			is_unisolated_in_cut_cut_die := b
		end

	set_is_isolating_cut (b: BOOLEAN)
			-- Set `is_isolating_cut' to `b'
		do
			is_isolating_cut := b
		end

	set_scribe_cut_die (a_cut_die: like scribe_cut_die)
			-- Set `scribe_cut_die' to `a_cut_die'
		do
		end

	set_edge_cut_die (a_cut_die: like edge_cut_die)
			-- Set `edge_cut_die' to `a_cut_die'
		do
		end

	set_cut_cut_die (a_cut_die: like cut_cut_die)
			-- Set `cut_cut_die' to `a_cut_die'
		do
		end

	set_has_surrounding_cuts (a_value: like has_surrounding_cuts)
			-- Set `has_surrounding_cuts' to `a_value'
		do
			has_surrounding_cuts := a_value
		ensure
			has_surrounding_cuts_set: has_surrounding_cuts = a_value
		end

	set_is_hidden (a_value: like is_hidden)
			-- Set `is_hidden' to `a_value'
		do
			is_hidden := a_value
		ensure
			is_hidden_set: is_hidden = a_value
		end

	set_is_cut_too_close_to_adjacent_cut (a_value: BOOLEAN)
			-- Set `is_cut_too_close_to_adjacent_cut' to `a_value'
		do
			is_cut_too_close_to_adjacent_cut := a_value
		ensure
			value_set: is_cut_too_close_to_adjacent_cut = a_value
		end

	set_is_cut_too_close_to_adjacent_strip_cut (a_value: BOOLEAN)
			-- Set `is_cut_too_close_to_adjacent_strip_cut' to `a_value'
		do
			is_cut_too_close_to_adjacent_strip_cut := a_value
		ensure
			value_set: is_cut_too_close_to_adjacent_strip_cut = a_value
		end

feature -- Operations

	set_view_coordinates
			-- Set `width', `height', `x', `y' and
			-- `bounding_box' based on size, location and
			-- rotation of `model_element' and its
			-- position on `view'
		do
			if has_sub_elements then
				from
					sub_elements.start
				until
					sub_elements.after
				loop
					sub_elements.item.set_view_coordinates
					sub_elements.forth
				end
			end
			unload_view_coordinates
		end

	set_selected (a_new_value, a_filter_matches: BOOLEAN)
			-- Set "selected" status of `Current' to
			-- `a_new_value' if `a_filter_matches' and
			-- item is selectable.  Update changed region
			-- and add `Current' to appropriate temporary
			-- list for maintaining element stacking order
		do
			if is_selectable and a_filter_matches then
				set_is_selected (a_new_value)
				add_to_region_if_selected_changed
			end
			if not is_selectable then
				unselectable_elements.extend (Current)
			elseif not is_selected then
				unselected_elements.extend (Current)
			elseif a_filter_matches then
				newly_selected_elements.extend (Current)
			else
				previously_selected_elements.extend (Current)
			end
		end

	add_to_new_element_list
			-- Add new element `Current' to list of new
			-- elements to be added to the view
		do
			view.add_to_new_element_list (Current)
		end

	add_to_region_if_selected_changed
			-- Add `Current' to the region to be updated if
			-- its "selected" status changed
		do
			if is_selected_changed then
				add_to_changed_region
			end
		end

	add_to_changed_region
			-- Add rectangle on canvas for element's bounding
			-- box to changed region, if it is not already there
		do
			if not included_in_changed_region then
				included_in_changed_region := True
			end
		end

	add_old_and_new_to_changed_region
			-- Add rectangle on canvas for element's current
			-- bounding box to changed region, recompute its
			-- new bounding box (after move or rotation) and
			-- add a rectangle for new bounding box to changed
			-- region
		do
			if not included_in_changed_region then
				set_view_coordinates
			end
		end

	add_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Add `a_change' to current changes
		require
			a_change_not_void: a_change /= Void
		do
			view.add_change (a_change)
		end

	add_prepended_change (a_change: RETICLE_SIMPLE_CHANGE)
			-- Add `a_change' to list of changes
			-- to be prepended to `current_changes' later
		require
			a_change_not_void: a_change /= Void
		do
			view.add_prepended_change (a_change)
		end

	move_after_rotation (dist: DOUBLE)
			-- Move element by `dist' in both
			-- X and Y if `dist' is nonzero
		local
			p: CONST_POINT
		do
			if dist /= 0.0 then
				create p.make_xy (dist, dist)
				move_by (p)
				add_change (create {ROTATION_POSITION_CHANGE}.make (Current, p))
				add_prepended_change (create {CENTER_ADJUSTMENT_CHANGE}.make (Current))
			end
		end

	adjust_center
			-- Adjust center of element, if necessary,
			-- so that edges remain on grid
		require
			is_abstract_die: is_abstract_die
		do
		end

	try_to_toggle_rotation
			-- Toggle rotation of element, if possible.
			-- Otherwise, add element to appropriate list
		local
			old_rot, new_rot: INTEGER
			dist: DOUBLE
		do
			verify_rotation_allowed
			if last_rotation_ok then
				if not is_processed then
					old_rot := rotation
					toggle_rotation
					new_rot := rotation
					set_processed
					add_change (create {ORIENTATION_CHANGE}.make (Current, old_rot, new_rot))
					dist := move_distance_for_rotation_change (old_rot, new_rot)
					set_rotation_move_distance (dist)
					move_after_rotation (dist)
				else
					-- Distance previously set when first
					-- copy of element was processed
					dist := rotation_move_distance
					move_after_rotation (dist)
				end
				add_old_and_new_to_changed_region
			end
			include_in_selection_bounding_box
		end

	try_to_rotate_90_clockwise
			-- Rotate element 90 degrees clockwise, if possible.
			-- Otherwise, add element to appropriate list
		local
			old_rot, new_rot: INTEGER
			dist: DOUBLE
		do
			verify_rotation_allowed
			if last_rotation_ok then
				if not is_processed then
					old_rot := rotation
					rotate_90_clockwise
					new_rot := rotation
					set_processed
					add_change (create {ORIENTATION_CHANGE}.make (Current, old_rot, new_rot))
					dist := move_distance_for_rotation_change (old_rot, new_rot)
					set_rotation_move_distance (dist)
					move_after_rotation (dist)
				else
					-- Distance previously set when first
					-- copy of element was processed
					dist := rotation_move_distance
					move_after_rotation (dist)
				end
				add_old_and_new_to_changed_region
			end
			include_in_selection_bounding_box
		end

	add_delete_error
			-- Add error for element that cannot be deleted
		do
			if not is_deletable then
				view.delete_locked_elements.extend (Current)
				add_error ("Cannot delete " + identification_for_message + " because it is delete-locked")
			else
				view.delete_failed_elements.extend (Current)
				if is_reworkable then
					add_error ("Cannot delete " + identification_for_message + " because it is in rework")
				else
					add_error ("Cannot delete " + identification_for_message + " because it is either locked and assigned to another run, has shipments or a database row lock failure occurred")
				end
			end
		end

	update_move_limits
			-- Update view distance limits for Move operation
		local
			up, down, left, right: DOUBLE
		do
			if is_locked_in_x then
				left := 0.0
				right := 0.0
			else
			end
			if is_locked_in_y then
				up := 0.0
				down := 0.0
			else
			end
		end

	update_duplicate_limits
			-- Update view distance limits for Duplicate operation
		local
			up, down, left, right: DOUBLE
		do
			if not is_duplicatable then
				left := 0.0
				right := 0.0
				up := 0.0
				down := 0.0
			else
			end
		end

	update_duplicate_lists
			-- Update view lists with elements that could not
			-- be duplicated for various reasons
		do
			if is_never_duplicatable then
				view.add_never_duplicatable_element (Current)
				add_warning ("Cannot duplicate " + identification_for_message + " because it is never duplicatable")
			elseif not is_duplicatable then
				view.add_duplication_locked_element (Current)
				add_warning ("Cannot duplicate " + identification_for_message + " because it is duplication-locked")
			end
		end

	duplicate
			-- Duplicate `Current' and put new copy in
			-- `last_duplicated_element'
		require
			is_abstract_die: is_abstract_die
		do
			last_duplicated_element.copy_display_attributes (Current)
		end

	update_resize_limits (a_left_resize, a_right_resize, a_bottom_resize, a_top_resize: BOOLEAN)
			-- Update view distance limits for Resize
			-- operation, where `a_left_resize' indicates whether
			-- we are resizing the left edge, etc
		local
			up, down, left, right, db_max: DOUBLE
		do
			up := 0.0
			down := 0.0
			left := 0.0
			right := 0.0
			db_max := max_db_value
			if a_left_resize then
			end
			if a_right_resize then
			end
			if a_bottom_resize then
			end
			if a_top_resize then
			end
		end

	resize_by (a_resize_amount: CONST_POINT is_left_resize, is_right_resize, is_bottom_resize, is_top_resize: BOOLEAN)
			-- Resize `Current' by `a_resize_amount', where
			-- `is_left_resize' indicates whether the left edge
			-- edge is to be moved by `a_resize_amount' X coordinate
			-- etc.
		require
			a_resize_amount_not_void: a_resize_amount /= Void
			is_resizable: is_resizable
		do
			if is_left_resize then
			end
			if is_right_resize then
			end
			if is_bottom_resize then
			end
			if is_top_resize then
			end
		end

	resize_scribe_by (a_resize_amount: CONST_POINT is_left_or_bottom: BOOLEAN)
			-- Resize `Current' by `a_resize_amount', where
			-- `is_left_or_bottom' indicates whether the left or
			-- bottom edge is to be moved by `a_resize_amount'
			-- (as opposed to right or top edge)
		require
			a_resize_amount_not_void: a_resize_amount /= Void
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				if is_left_or_bottom then
				else
				end
			else
				if is_left_or_bottom then
				else
				end
			end
		end

	change_scribe_width (a_low_change, a_high_change: DOUBLE)
			-- Change width of `Current' by moving low
			-- edge (left or bottom edge) by
			-- `a_low_change' and high edge (right or top
			-- edge) by `a_high_change'
		require
			is_scribe: is_scribe
		do
			if is_horizontal_scribe then
				if a_low_change /= 0.0 then
				end
				if a_high_change /= 0.0 then
				end
			else
				if a_low_change /= 0.0 then
				end
				if a_high_change /= 0.0 then
				end
			end
		end

	delete
			-- Delete `Current' when not all copies are selected
			-- for a Delete operation
		require
			is_die: is_die
		do
			remove_from_all_copies
		end

	isolate
			-- Isolate `Current' in its cut die in current
			-- cut pattern
		require
			is_die: is_die
		do
			view.isolate (Current)
		end

	unisolate
			-- Remove `Current' from list of isolated die
			-- locations in current cut pattern
		require
			is_die: is_die
		do
			view.unisolate (Current)
		end

	set_isolated_site_cut_die
			-- Set cut die size and center for `an_element'
			-- in current cut pattern
		require
			is_die: is_die
		do
			view.set_isolated_site_cut_die (Current)
		end

	copy_display_attributes (an_element: like Current)
			-- Copy attributes which affect how `Current' is
			-- displayed from `an_element'
		do
			set_view_coordinates
		end

	prepare_for_horizontal_spread
			-- Take actions needed before a horizontal spread
			-- operation
		do
			view.add_horizontal_spread_element (Current)
		end

	prepare_for_vertical_spread
			-- Take actions needed before a vertical spread
			-- operation
		do
			view.add_vertical_spread_element (Current)
		end

	check_move_restrictions (a_distance: POINT)
			-- Check that element can be moved `a_distance'
		require
			a_distance_not_void: a_distance /= Void
		do
			if (is_locked_in_x and a_distance.x /= 0.0) or (is_locked_in_y and a_distance.y /= 0.0) then
			end
		end

	check_outside_fields
			-- Check whether element is completely outside any
			-- field of view
		do
			if not intersects_any_field then
				add_error (identification_for_message + " is outside all fields of view " + location_description)
			end
		end

	check_overlapping_field (scribes_included: BOOLEAN)
			-- Check whether element intersects at least one
			-- field of view but is not enclosed by any field
		do
			if partially_outside_fields and (scribes_included or is_non_scribe) then
				add_error (identification_for_message + " overlaps field edge " + location_description)
			end
		end

	check_overlapping_object (scribes_included: BOOLEAN)
			-- Check whether element intersects at least one
			-- field of view and overlaps any other object that
			-- intersects at least one field of view.
			-- Set `is_overlapping_another_object' for Validity
			-- Check view coloring
		local
			areas: LIST [RETICLE_MODEL_AREA]
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_mbb_areas
				from
					areas.start
				until
					areas.after
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after
					loop
						elem := elems.item
						elems.forth
					end
					areas.forth
				end
				mark
			end
		end

	check_too_close_to_object (scribes_included: BOOLEAN)
			-- Check whether element intersects at least one
			-- field of view and is too close to (but does not
			-- overlap) any other object that
			-- intersects at least one field of view.
			-- Set `is_too_close_to_other_object' for Validity
			-- Check view coloring
		local
			areas: LIST [RETICLE_MODEL_AREA]
			mbb: CONST_RECTANGLE
			bigger_mbb, other_mbb: RECTANGLE
			gap, other_grow, dist_x, dist_y: DOUBLE
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
		do
			if intersects_any_field then
				areas := intersecting_mbb_areas
				gap := minimum_gap
				if gap > 0.0 then
					create bigger_mbb.make_copy (mbb)
					bigger_mbb.size_by (2.0 * gap, 2.0 * gap)
					from
						areas.start
					until
						areas.after
					loop
						area := areas.item
						elems := area.intersecting_elements
						from
							elems.start
						until
							elems.after
						loop
							elem := elems.item
							if elem /= Current and then neither_is_partition (elem) and then not mbb.intersects (other_mbb) and (scribes_included or is_non_scribe or elem.is_non_scribe) then
								other_grow := -2.0 * elem.uniform_white_space_border
								other_mbb.size_by (other_grow, other_grow)
								dist_x := distance_in_x_from (elem)
								dist_y := distance_in_y_from (elem)
								if bigger_mbb.intersects (other_mbb) and (needs_min_sep_white_space or elem.needs_min_sep_white_space or dist_x /= 0.0 or dist_y /= 0.0) then
									is_too_close_to_another_object := True
									if not elem.is_marked and not invalid_elements.has (elem) then
										invalid_elements.extend (elem)
										report_too_close_to_object (elem, dist_x, dist_y, minimum_separation_from (elem), not needs_min_sep_white_space and not elem.needs_min_sep_white_space)
									end
								end
							end
							elems.forth
						end
						areas.forth
					end
				end
				mark
			end
		end

	check_misaligned_scribe
			-- Check whether element intersects at least
			-- one field of view and is misaligned with
			-- another scribe of same orientation that
			-- intersects at least one field of view.  Set
			-- `is_misaligned_scribe' for Validity Check
			-- view coloring
		local
			areas: LIST [RETICLE_MODEL_AREA]
			area: RETICLE_MODEL_AREA
			elems: LIST [RETICLE_VIEW_ELEMENT]
			elem: RETICLE_VIEW_ELEMENT
			diff: DOUBLE
		do
			if intersects_any_field then
				areas := intersecting_scribe_areas
				from
					areas.start
				until
					areas.after
				loop
					area := areas.item
					elems := area.intersecting_elements
					from
						elems.start
					until
						elems.after
					loop
						elem := elems.item
						if elem /= Current and then is_misaligned_with (elem) then
							is_misaligned_scribe := True
							if not elem.is_marked and not invalid_elements.has (elem) then
								invalid_elements.extend (elem)
								diff := misalignment_amount (elem)
								add_warning (identification_for_message + " is misaligned with " + elem.identification_for_message + " by " + grid.formatted_number (diff) + "%N" + Indent + identification_for_message + " is " + location_description + "%N" + Indent + elem.identification_for_message + " is " + elem.location_description)
							end
						end
						elems.forth
					end
					areas.forth
				end
				mark
			end
		end

	check_wrong_rotation
			-- Check whether element has wrong rotation
			-- (doesn't match required rotation)
		do
			if has_wrong_rotation and not is_processed then
				set_processed
				add_error (identification_for_message + " rotation " + rotation.out + " does not match required rotation " + required_rotation.out)
			end
		end

	check_below_minimum_size
			-- Check whether element is below minimum valid size
			-- in either dimension
		do
			if is_below_minimum_size and not is_processed then
				set_processed
			end
		end

	check_no_lot_assigned
			-- Check whether element is a project with no lot
			-- assigned
		do
			if is_wafer_etest_id_missing and not is_processed then
				set_processed
				add_error (identification_for_message + " has not been assigned to a lot split")
			end
		end

	check_unlocked
			-- Check whether element is a project which is
			-- not locked against customer update
		do
			if is_cust_project and is_unlocked_project and not is_processed then
				set_processed
				add_warning (identification_for_message + " " + "is not locked against customer update")
			end
		end

	check_unassigned
			-- Check whether element is a project which is
			-- not assigned to run being closed
		do
			if is_cust_project and is_unassigned_project and not is_processed then
				set_processed
				add_error (identification_for_message + " " + "is not assigned to this run")
			end
		end

	check_unlocked_or_unassigned
			-- Check whether element is a project which is not
			-- locked against customer update or not assigned to
			-- run being closed
		do
			if is_cust_project and not is_processed then
				set_processed
				if is_unassigned_project then
					add_error (identification_for_message + " " + "is not assigned to this run")
				end
				if is_unlocked_project then
					add_warning (identification_for_message + " " + "is not locked against customer update")
				end
			end
		end

	check_too_close_to_edge (scribes_included: BOOLEAN)
			-- Check whether element is too close to a field edge
		local
			l, r, t, b: DOUBLE
			l_bad, r_bad, t_bad, b_bad: BOOLEAN
		do
			if scribes_included or is_non_scribe then
				l := left_to_field_edge
				r := right_to_field_edge
				t := top_to_field_edge
				b := bottom_to_field_edge
				l_bad := is_edge_too_close_to_field_edge (l)
				r_bad := is_edge_too_close_to_field_edge (r)
				t_bad := is_edge_too_close_to_field_edge (t)
				b_bad := is_edge_too_close_to_field_edge (b)
				if l_bad then
					report_too_close_to_edge ("left", l, minimum_gap, not needs_min_sep_white_space)
				end
				if r_bad then
					report_too_close_to_edge ("right", r, minimum_gap, not needs_min_sep_white_space)
				end
				if t_bad then
					report_too_close_to_edge ("top", t, minimum_gap, not needs_min_sep_white_space)
				end
				if b_bad then
					report_too_close_to_edge ("bottom", b, minimum_gap, not needs_min_sep_white_space)
				end
			end
		end

	check_scribe_cut_die_too_big
			-- Check whether element has a cut die size
			-- determined by surrounding scribes that
			-- exceeds its maximum die size
		local
			l_user_max: like user_max_die_size
			l_pkg_max: like packaging_max_die_size
			msg: STRING
		do
			l_user_max := user_max_die_size
			if is_scribe_cut_die_too_big_for (l_user_max) then
				msg := identification_for_message + " cut die size " + scribe_cut_size_description + " exceeds user-specified maximum size of " + grid.formatted_number (l_user_max.x) + " x " + grid.formatted_number (l_user_max.y) + "%N    " + identification_for_message + " is " + location_description
				if is_oversize_die_always_allowed then
					add_warning (msg)
				elseif is_oversize_die_possibly_allowed then
					add_warning (msg + oversize_inclusion_text)
				else
					add_error (msg)
				end
			end
			
			l_pkg_max := packaging_max_die_size
			if is_scribe_cut_die_too_big_for (l_pkg_max) then
				msg := identification_for_message + " cut die size " + scribe_cut_size_description + " exceeds package-constrained maximum size of " + grid.formatted_number (l_pkg_max.x) + " x " + grid.formatted_number (l_pkg_max.y) + "%N    " + identification_for_message + " is " + location_description
				add_error (msg)
			end
		end

	check_unisolated_in_scribe_cut_die
			-- Check whether element is not isolated in its cut die
			-- determined by surrounding scribes
		do
			if unisolated_in_scribe_cut_die then
				is_unisolated_in_scribe_cut_die := True
				add_warning (identification_for_message + " is not isolated in its cut die " + location_description)
			end
		end

	compute_unisolated_in_scribe_cut_die
			-- Compute whether element is not isolated in its
			-- cut die determined by surrounding scribes
		do
			if unisolated_in_scribe_cut_die then
				is_unisolated_in_scribe_cut_die := True
			end
		end

	check_uncentered_in_cut_die
			-- Check whether element is not centered in its cut die
		do
			if is_uncentered_in_cut_die then
				add_warning (identification_for_message + " is offset from cut die center by " + offset_cut_center_description + " " + location_description)
			end
		end

	check_invalid_cut_die
			-- Check whether element is isolated in current cut
			-- pattern but has an invalid cut die determined by
			-- surrounding cuts
		local
			what: STRING
		do
			if is_cut_cut_die_invalid then
				if has_surrounding_cuts then
					what := "is isolated but has a cut through printed die"
				else
					what := "is isolated but is not surrounded by cuts on all sides"
				end
				add_error (identification_for_message + " " + what + " " + location_description)
			end
		end

	check_cut_cut_die_too_big
			-- Check whether element has a cut die size
			-- determined by surrounding cuts that
			-- exceeds its maximum die size
		local
			l_user_max: like user_max_die_size
			l_pkg_max: like packaging_max_die_size
			msg: STRING
		do
			l_user_max := user_max_die_size
			if is_cut_cut_die_too_big_for (l_user_max) then
				msg := identification_for_message + " cut die size " + cut_cut_size_description + " exceeds user-specified maximum size of " + grid.formatted_number (l_user_max.x) + " x " + grid.formatted_number (l_user_max.y) + "%N    " + identification_for_message + " is " + location_description
				if is_oversize_die_always_allowed then
					add_warning (msg)
				elseif is_oversize_die_possibly_allowed then
					add_warning (msg + oversize_inclusion_text)
				else
					add_error (msg)
				end
			end
			
			l_pkg_max := packaging_max_die_size
			if is_cut_cut_die_too_big_for (l_pkg_max) then
				msg := identification_for_message + " cut die size " + cut_cut_size_description + " exceeds package-constrained maximum size of " + grid.formatted_number (l_pkg_max.x) + " x " + grid.formatted_number (l_pkg_max.y) + "%N    " + identification_for_message + " is " + location_description
				add_error (msg)
			end
		end

	check_wrong_lot
			-- Check whether element is isolated and has the
			-- wrong lot (different from the one for the cut
			-- pattern)
		do
			if has_wrong_lot_for_cut_pattern then
			end
		end

	check_non_default_cut_die_size
			-- Check whether element that is isolated has
			-- an cut die size determined by surrounding
			-- cuts that is different from its default cut
			-- die size determined by surrounding scribes
		do
			if has_non_default_cut_die_size then
				add_warning (identification_for_message + " has cut die that differs from cut die determined by surrounding scribes" + "%N" + Indent + "Cut cut die size is " + grid.formatted_point (oriented_cut_cut_die.size) + "%N" + Indent + "Scribe cut die size is " + grid.formatted_point (oriented_scribe_cut_die.size) + "%N" + Indent + identification_for_message + " is " + location_description)
			end
		end

	check_shared_cut_die
			-- Check whether element that is isolated
			-- shares its cut die partially or fully with
			-- one or more other non-scribe die elements
		do
				is_unisolated_in_cut_cut_die := True
				add_warning (identification_for_message + " shares in its cut die with others " + location_description)
		end

	check_extra_cut
			-- Check whether element is an extra cut that does
			-- not help isolate any isolated sites
			-- one or more other non-scribe die elements
		do
			if is_extra_cut then
				add_warning (identification_for_message + " is an extra cut that does not help isolate any isolated sites " + location_description)
			end
		end

	check_cut_to_cut_distance (prev_cut: like Current min_dist: DOUBLE)
			-- Check whether `prev_cut' is closer than
			-- `min_dist' to `Current'
		require
			is_cut: is_cut
			prev_cut_no_Void: prev_cut /= Void
			prev_cut_is_cut: prev_cut.is_cut
			same_orientation: is_horizontal_cut and prev_cut.is_horizontal_cut or is_vertical_cut and prev_cut.is_vertical_cut
			min_dist_positive: min_dist > 0
		local
			dist: DOUBLE
		do
			dist := grid.rounded_center (cut_position - prev_cut.cut_position) 
			if dist < min_dist then
				set_is_cut_too_close_to_adjacent_cut (True)
				prev_cut.set_is_cut_too_close_to_adjacent_cut (True)
				add_error (identification_for_message + " is too close to adjacent cut - distance is " + grid.formatted_number (dist) + " but minimum distance is " + grid.formatted_number (min_dist) + "%N" + Indent + identification_for_message + " is " + location_description + "%N" + Indent + "Adjacent " + prev_cut.identification_for_message + " is " + prev_cut.location_description)
			end
		end

	check_cut_to_strip_cut_distance (prev_cut: like Current min_strip_width: DOUBLE)
			-- Check whether `prev_cut' is closer than
			-- `min_strip_width' to `Current' and therefore
			-- produces a strip that is narrower tha
			-- the minimum width
		require
			is_cut: is_cut
			prev_cut_no_Void: prev_cut /= Void
			prev_cut_is_cut: prev_cut.is_cut
			same_orientation: is_horizontal_cut and prev_cut.is_horizontal_cut or is_vertical_cut and prev_cut.is_vertical_cut
			min_strip_width_positive: min_strip_width > 0
		local
			dist: DOUBLE
		do
			dist := grid.rounded_center (cut_position - prev_cut.cut_position) 
			if dist < min_strip_width then
				set_is_cut_too_close_to_adjacent_strip_cut (True)
				prev_cut.set_is_cut_too_close_to_adjacent_strip_cut (True)
				add_warning (identification_for_message + " is too close to adjacent cut to make a valid wafer strip - distance is " + grid.formatted_number (dist) + " but minimum strip width is " + grid.formatted_number (min_strip_width) + "%N" + Indent + identification_for_message + " is " + location_description + "%N" + Indent + "Adjacent " + prev_cut.identification_for_message + " is " + prev_cut.location_description)
			end
		end

	check_empty_partition
			-- Check whether element intersects at least
			-- one field of view and is an empty partition
			-- (one that does not contain any cust projects
			-- or test vehicles).  Set
			-- `is_empty_partition' for Validity Check
			-- view coloring
		do
			if is_partition and intersects_any_field and not contains_project then
				is_empty_partition := True
				add_error (identification_for_message + " is an empty partition " + "%N" + Indent + identification_for_message + " is " + location_description)
			end
		end

	set_invalid_partition_edge_position (bad_left, bad_right, bad_bottom, bad_top: BOOLEAN)
			-- Set `is_partition_with_invalid_edge' for
			-- Validity Check view coloring and add error
			-- message to error list
		do
			is_partition_with_invalid_edge := True
			add_error (identification_for_message + " is a partition with an edge in an invalid position (not on field edge or near edge of scribe)" + "%N" + Indent + "Invalid edge position on " + invalid_edge_position_text (bad_left, bad_right, bad_bottom, bad_top) + "%N" + Indent  + identification_for_message + " is " + location_description)
		end

	check_not_contained_in_partition (list: LIST [RETICLE_VIEW_ELEMENT])
			-- Check whether element is fully contained in
			-- at least one of the elements in `list'.
			-- Set `is_not_contained_in_partition' for
			-- Validity Check view coloring
		do
			if intersects_any_field and not is_contained_in_any_element (list) then
				is_not_contained_in_partition := True
				add_error (identification_for_message + " is not completely inside any partition" + "%N" + Indent + identification_for_message + " is " + location_description)
			end
		end

	check_partially_in_partition (list: LIST [RETICLE_VIEW_ELEMENT])
			-- Check whether element intersects at least
			-- one of the elements of `list' but is not
			-- fully contained in any of them.  Set
			-- `is_partially_in_partition' for Validity
			-- Check view coloring
		do
			if intersects_any_field and intersects_any_element (list) and not is_contained_in_any_element (list) then
				is_partially_in_partition := True
				add_error (identification_for_message + " intersects a partition but is not completely inside any partition" + "%N" + Indent + identification_for_message + " is " + location_description)
			end
		end

	check_invalid_scribe_placement (list: LIST [RETICLE_VIEW_ELEMENT])
			-- Check whether element intersects none of
			-- the elements of `list' and abuts a field
			-- edge on its length side.  Set
			-- `is_invalidly_placed_scribe' for Validity
			-- Check view coloring
		require
			is_scribe: is_scribe
		do
			if intersects_any_field and not intersects_any_element (list) then
				if abuts_field_edge_on_length_side then
					is_invalidly_placed_scribe := True
					add_error (identification_for_message + " is outside all partitions and its length side abuts field of view edge" + "%N" + Indent + identification_for_message + " is " + location_description)
				elseif not abuts_element_on_both_sides (list) then
					is_invalidly_placed_scribe := True
					add_error (identification_for_message + " is outside all partitions and does not abut a partition on both length sides" + "%N" + Indent + identification_for_message + " is " + location_description)
				end
			end
		end

	check_wrong_lot_for_partition (list: LIST [RETICLE_VIEW_ELEMENT])
			-- Check whether element center is contained
			-- in exactly one partition in `list' but
			-- has a different assigned lot than the
			-- one for its partition.  Set
			-- `has_wrong_lot_for_partition' for Validity
			-- Check view coloring
		local
			l_partition: RETICLE_VIEW_ELEMENT
			msg: STRING
		do
			if intersects_any_field then
				l_partition := unique_containing_element (list)
				if l_partition /= Void and then mismatches_partition_lot (l_partition) then
					has_wrong_lot_for_partition := True
					msg := identification_for_message + " is assigned to lot " + assigned_wafer_etest_id + " but containing partition is for lot " + l_partition.constraining_wafer_etest_id + "%N" + Indent + identification_for_message + " is " + location_description
					if all_compatible_etest_names_manufacturable and not any_compatible_etest_name_matches (l_partition) then
						add_error (msg)
					else
						add_warning (msg)
					end
				end
			end
		end

	set_floating_scribe
			-- Set element to be a floating scribe and add
			-- warning
		do
			set_is_floating_scribe (True)
			add_warning (identification_for_message + " is a floating scribe " + location_description)
		end

	set_misaligned_scribe
			-- Set element to be a misaligned scribe and add
			-- warning
		do
			set_is_misaligned_scribe (True)
			add_warning (identification_for_message + " is a misaligned scribe " + location_description)
		end

	report_small_but_legal (scribes_included: BOOLEAN)
			-- Report element as small but legal
		do
			if scribes_included or is_non_scribe then
				-- Report all locations 
				-- (ignore is_processed indicator)
				set_processed
				add_warning (identification_for_message + " size " + size_description + " is legal but small " + location_description)
			end
		end

	add_field
			-- Add `Current' to list of fields
		do
		end

	add_error (a_message: STRING)
			-- Add a new error described by `a_message'
		do
		end

	add_warning (a_message: STRING)
			-- Add a new warning described by `a_message'
		do
		end

	set_processed
			-- Set indicator that a copy of element has been
			-- processed
		do
			all_copies.set_processed
		end

	clear_processed
			-- Clear indicator that a copy of element has been
			-- processed
		do
			all_copies.clear_processed
		end

	set_rotation_move_distance (d: DOUBLE)
			-- Set distance element had
			-- to be moved in X and Y after
			-- rotation to keep edges on grid
		do
			all_copies.set_rotation_move_distance (d)
		end

	uncache_all_copies_selected
			-- Remove from cache value computed for
			-- `all_copies_selected'
		do
			all_copies.uncache_all_elements_selected
		end

	uncache_any_copies_selected
			-- Remove from cache value computed for
			-- `any_copies_selected'
		do
			all_copies.uncache_any_elements_selected
		end

	uncache_any_copies_stationary_orientation
			-- Remove from cache value computed for
			-- `any_copies_stationary_orientation'
		do
			all_copies.uncache_any_elements_stationary_orientation
		end

	uncache_any_copies_rotation_locked
			-- Remove from cache value computed for
			-- `any_copies_rotation_locked'
		do
			all_copies.uncache_any_elements_rotation_locked
		end

	uncache_any_copies_contain
			-- Remove from cache value computed for
			-- `any_copies_contain'
		do
			all_copies.uncache_any_elements_contain
		end

	include_in_selection_bounding_box
			-- Include element in current selection bounding box
		do
			view.include_in_selection_bounding_box (Current)
		end

	mark
			-- Set "marked" indicator
		do
			is_marked := True
		end

	unmark
			-- Clear "marked" indicator
		do
			is_marked := False
		end

	clear_invalid_elements
			-- Clear list of invalid elements with which
			-- `Current' has some kind of validity problem
		do
			create {ARRAYED_LIST [RETICLE_VIEW_ELEMENT]} invalid_elements.make (5)
			invalid_elements.compare_references
		end

	set_is_floating_scribe (b: BOOLEAN)
			-- Set `is_floating_scribe' to `b'
		do
			is_floating_scribe := b
		ensure
			value_set: is_floating_scribe = b
		end

	set_is_misaligned_scribe (b: BOOLEAN)
			-- Set `is_misaligned_scribe' to `b'
		do
			is_misaligned_scribe := b
		ensure
			value_set: is_misaligned_scribe = b
		end

	set_is_empty_partition (b: BOOLEAN)
			-- Set `is_empty_partition' to `b'
		do
			is_empty_partition := b
		ensure
			value_set: is_empty_partition = b
		end

	set_is_partition_with_invalid_edge (b: BOOLEAN)
			-- Set `is_partition_with_invalid_edge' to `b'
		do
			is_partition_with_invalid_edge := b
		ensure
			value_set: is_partition_with_invalid_edge = b
		end

	set_is_not_contained_in_partition (b: BOOLEAN)
			-- Set `is_not_contained_in_partition' to `b'
		do
			is_not_contained_in_partition := b
		ensure
			value_set: is_not_contained_in_partition = b
		end

	set_is_partially_in_partition (b: BOOLEAN)
			-- Set `is_partially_in_partition' to `b'
		do
			is_partially_in_partition := b
		ensure
			value_set: is_partially_in_partition = b
		end

	set_is_invalidly_placed_scribe (b: BOOLEAN)
			-- Set `is_invalidly_placed_scribe' to `b'
		do
			is_invalidly_placed_scribe := b
		ensure
			value_set: is_invalidly_placed_scribe = b
		end

	set_has_wrong_lot_for_partition (b: BOOLEAN)
			-- Set `has_wrong_lot_for_partition' to `b'
		do
			has_wrong_lot_for_partition := b
		ensure
			value_set: has_wrong_lot_for_partition = b
		end

	initialize_intersecting_mbb_areas (n: INTEGER)
			-- Create `intersecting_mbb_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_mbb_areas.make (n)
		end

	initialize_intersecting_scribe_cut_die_areas (n: INTEGER)
			-- Create `intersecting_scribe_cut_die_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_scribe_cut_die_areas.make (n)
		end

	initialize_intersecting_cut_cut_die_areas (n: INTEGER)
			-- Create `intersecting_cut_cut_die_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_cut_cut_die_areas.make (n)
		end

	initialize_intersecting_hscribe_areas (n: INTEGER)
			-- Create `intersecting_hscribe_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_hscribe_areas.make (n)
		end

	initialize_intersecting_vscribe_areas (n: INTEGER)
			-- Create `intersecting_vscribe_areas' with capacity `n'
		do
			create {ARRAYED_LIST [RETICLE_MODEL_AREA]} intersecting_vscribe_areas.make (n)
		end

	populate_intersecting_mbb_areas
			-- Populate `intersecting_mbb_areas' and add element
			-- to intersecting elements list of every model
			-- area its bounding box intersects
		do
		end

	populate_intersecting_scribe_cut_die_areas
			-- Populate `intersecting_scribe_cut_die_areas' and
			-- add element to intersecting elements list
			-- of every model area its cut die (determined by
			-- surrounding scribes) intersects
		do
		end

	populate_intersecting_cut_cut_die_areas
			-- Populate `intersecting_cut_cut_die_areas' and
			-- add element to intersecting elements list
			-- of every model area its cut die (determined by
			-- surrounding cuts) intersects.  Use cut die
			-- determined by surrounding scribes if cut cut
			-- die is Void
		do
		end

	populate_intersecting_hscribe_areas
			-- Populate `intersecting_hscribe_areas' and
			-- add element to intersecting elements list
			-- of every model area its bounding box
			-- intersects
		do
		end

	populate_intersecting_vscribe_areas
			-- Populate `intersecting_vscribe_areas' and
			-- add element to intersecting elements list
			-- of every model area its bounding box
			-- intersects
		do
		end

	populate_intersecting_areas (areas: LIST [RETICLE_MODEL_AREA]  mbb: CONST_RECTANGLE intersecting_areas: like intersecting_mbb_areas)
			-- Populate `intersecting_areas' by adding
			-- element to intersecting elements list of
			-- every model area in `areas' that `mbb'
			-- intersects
		do
		end

	increment_centered_count
			-- Increment count of projects centered in cut die
			-- by Center operation
		do
		end

	report_isolation_failure
			-- Add error message when toggling isolation of
			-- project is not allowed
		do
			if has_surrounding_cuts then
				add_error (identification_for_message + " has a cut through the printed die " + location_description)
			else
				add_error (identification_for_message + " is not surrounded by cuts on all sides " + location_description)
			end
		end

feature {NONE} -- Implementation

	verify_rotation_allowed
			-- Verify that element can be rotated and add
			-- warning if not.  Set `last_rotation_ok' to
			-- indicate whether rotation is allowed
		do
			last_rotation_ok := False
			if any_copies_rotation_locked then
				view.add_rotation_locked_element (Current)
				add_rotate_warning ("Cannot rotate " + identification_for_message + " because at least one copy is rotation-locked")
			elseif is_stationary_orientation then
				view.add_fixed_orientation_element (Current)
				add_rotate_warning ("Cannot rotate " + identification_for_message + " because its orientation is fixed")
			elseif not is_rotatable then
			elseif not all_copies_selected then
				view.add_partially_selected_element (Current)
				add_rotate_warning ("Cannot rotate " + identification_for_message + " because not all copies are selected")
			else
				last_rotation_ok := True
			end
		end

	move_distance_for_rotation_change (old_rot, new_rot: INTEGER): DOUBLE
			-- Distance element must be
			-- moved (in both X and Y) after
			-- being rotated from `old_rot' to
			-- `new_rot', to keep edges on grid
		do
			if (grid.rounded_size_units (size_x) \\ 2) /= (grid.rounded_size_units (size_y) \\ 2) then
				-- Element size is an even number of
				-- grid units in one dimension and an
				-- odd number of grid units in the
				-- other dimension
				if (old_rot \\ 180) = (new_rot \\ 180) then
					-- No movement required
					Result := 0.0
				elseif new_rot \\ 180 = 90 then
					-- Move up and to right 1/2 grid unit
					Result := grid.edge_resolution / 2.0
				elseif new_rot \\ 180 = 0 then
					-- Move down and to left 1/2 grid unit
					Result := - (grid.edge_resolution / 2.0)
				else
				end
			else
				-- Element size in grid units is
				-- even or odd in both dimensions
				Result := 0.0
			end
		end

	add_rotate_warning (a_message: STRING)
			-- Add warning about rotation that failed
		do
			if not is_processed then
				add_warning (a_message)
				set_processed
			end
		end

	last_rotation_ok: BOOLEAN
			-- Was last attempt to rotate element allowed?

	anchored_view_sub_element: RETICLE_VIEW_SUB_ELEMENT
			-- Type of sub-elements in view
		do
		end

	unselectable_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of items that cannot be selected
		do
		end

	unselected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of items that are not selected but can be
			-- selected
		do
		end

	previously_selected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of items that are selected but which
			-- did not match the current filter that determines
			-- which items have their selection status changed
		do
		end

	newly_selected_elements: LIST [RETICLE_VIEW_ELEMENT]
			-- List of items that are selected and which
			-- matched the current filter that determines
			-- which items have their selection status changed
		do
		end

	report_too_close_to_edge (an_edge: STRING a_distance, a_minimum_gap: DOUBLE a_zero_ok: BOOLEAN)
			-- Add warning about element `an_edge' edge which
			-- is `a_distance' away from corresponding field
			-- edge (which is too close)
		local
			zero: STRING
		do
			if a_zero_ok then
				zero := "gap = 0 or "
			else
				zero := ""
			end
			add_warning (identification_for_message + " is too close to field " + an_edge + " edge (gap = " + grid.formatted_number (a_distance) + ", requires " + zero + " gap >= " + grid.formatted_number (a_minimum_gap) + ") " + location_description)
		end

	report_too_close_to_object (an_element: RETICLE_VIEW_ELEMENT a_distance_x, a_distance_y, a_minimum_gap: DOUBLE a_zero_ok: BOOLEAN)
			-- Add warning about element `an_element' edge which
			-- is `a_distance_x' and `a_distance_y' away from
			-- `Current', which is too close
		local
			zero: STRING
			dist: DOUBLE
		do
			if a_zero_ok then
				zero := "gap = 0 or "
			else
				zero := ""
			end
			if a_distance_x = 0.0 then
				dist := a_distance_y
			elseif a_distance_y = 0.0 then
				dist := a_distance_x
			else
				dist := a_distance_x.min (a_distance_y)
			end

			add_warning (identification_for_message + " is too close to " + an_element.identification_for_message + " (gap = " + grid.formatted_number (dist) + ", requires " + zero + " gap >= " + grid.formatted_number (a_minimum_gap) + ")%N" + Indent + identification_for_message + " is " + location_description + "%N" + Indent + an_element.identification_for_message + " is " + an_element.location_description)
		end

	oversize_inclusion_text: STRING
			-- Text to add to warning message if oversize
			-- die is to be included on run but only under
			-- certain conditions
		do
		end

	invalid_edge_position_text (bad_left, bad_right, bad_bottom, bad_top: BOOLEAN): STRING
			-- Text describing which edges have invalid
			-- positions
		do
			create Result.make (30)
			if bad_left then
				Result.append ("Left")
			end
			if bad_right then
				if not Result.is_empty then
				end
				Result.append ("Right")
			end
			if bad_bottom then
				if not Result.is_empty then
				end
				Result.append ("Bottom")
			end
			if bad_top then
				if not Result.is_empty then
				end
				Result.append ("Top")
			end
		end

	Max_inclusion_length: INTEGER = 100
			-- Maximum length of lines in `oversize_inclusion_text'

	Indent: STRING = "    "
			-- Indentation for non-first lines of a
			-- warning or error message

	
end
