
note
	
	description: "Object that can access information about scribe edges"
	author: "David Hollenberg"
	date: "January 17, 2007"

class SCRIBE_EDGE_ACCESSOR
inherit
	GRIDABLE

feature -- Properties

	field_list: LIST [FIELD_ELEMENT_EDGES]
			-- List of fields of view and their horizontal and
			-- vertical edges

feature -- Modification

	set_field_list (a_list: like field_list)
			-- Set `field_list' to `a_list'
		require
			a_list_not_void: a_list /= Void
		do
			field_list := a_list
		ensure
			field_list_set: field_list = a_list
		end

feature {NONE} -- Implementation

	is_floating_scribe (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is `an_element' a floating scribe?
		do
			if an_element.is_scribe and an_element.intersects_any_field then
				Result := is_floating (an_element)
			end
		end

	is_floating (an_element: RETICLE_VIEW_ELEMENT): BOOLEAN
			-- Is either end of `an_element' floating (not
			-- on a field of view edge of an edge of scribe
			-- of opposite orientation)?
		local
			low, high: DOUBLE
		do
			if an_element.is_horizontal_scribe then
				low := grid.rounded_edge (an_element.bottom_y)
				high := grid.rounded_edge (an_element.top_y)
			else
				low := grid.rounded_edge (an_element.left_x)
				high := grid.rounded_edge (an_element.right_x)
			end
		end

	meets_field_edge (is_horiz: BOOLEAN pos, low, high: DOUBLE is_low_end: BOOLEAN): BOOLEAN
			-- Does scribe end at `pos' which extends from
			-- `low' to `high' and is horizontal if `is_horiz' and
			-- is left or bottom end if `is_low_end' meet the
			-- edge of any field of view along entire length
			-- of end?
		do
		end

feature {NONE} -- Implementation

	closest_meetable_edge (pos, low, high: DOUBLE edges: LIST [MODEL_ELEMENT_EDGE] is_low_end, meetable_only: BOOLEAN): MODEL_ELEMENT_EDGE
			-- Closest edge to end of `an_element' at
			-- `pos' (which direction to search is
			-- indicated by `is_low_end' - bottom or left
			-- end if true) that truly intersects segment
			-- (`low', `high') or Void if closest edge is
			-- not meetable.  If `meetable_only' then
			-- ignore non-meetable edges when looking
			-- for the closest edge
		local
			e, closest_edge: MODEL_ELEMENT_EDGE
			done: BOOLEAN
			next_position, last_position: DOUBLE
		do
			if is_low_end then
				from
					done := False
					edges.start
					last_position := edges.item.position - 1.0E6
				until
					edges.after or done
				loop
					e := edges.item
					next_position := e.position
					done := next_position > pos
					if not done and (not meetable_only or e.is_meetable) and e.truly_intersects (low, high) then
						if next_position > last_position or else (next_position = last_position and e.is_meetable and (not closest_edge.is_meetable or else closest_edge.is_low_edge = is_low_end)) then
							-- Prefer meetable edge over non-meetable edge
							-- Prefer meetable edge with opposite "low edge"-ness
							-- to meetable edge with same "low edge"-ness
							-- (this prefers near edge of object over far edge
							-- of some other object)
							closest_edge := e
							last_position := next_position
						end
					end
					edges.forth
				end
				
				if closest_edge.is_meetable then
					Result := closest_edge
				end
			else
				from
					done := False
					edges.finish
					last_position := edges.item.position + 1.0E6
				until
					edges.before or done
				loop
					e := edges.item
					next_position := e.position
					done := next_position < pos
					if not done and (not meetable_only or e.is_meetable) and e.truly_intersects (low, high) then
						if next_position < last_position or else (next_position = last_position and e.is_meetable and (not closest_edge.is_meetable or else closest_edge.is_low_edge = is_low_end)) then
							closest_edge := e
							last_position := next_position
						end
					end
					edges.back
				end
				
				if closest_edge.is_meetable then
					Result := closest_edge
				end
			end
		end

end
