
note
	
	description: "Object that can resize an element by moving one side"
	author: "David Hollenberg"

deferred class SIDE_RESIZE_ACTION
inherit
	MULTIPLE_CHANGE_ACTION
		redefine
			default_create
		end
	GRIDABLE
		redefine
			default_create
		end

feature {NONE} -- Creation

	default_create
		do
			create resize_amount.make_xy (0.0, 0.0)
		end

feature -- Operations

	resize_one_side (an_element: RETICLE_VIEW_ELEMENT is_left, is_right, is_bottom, is_top: BOOLEAN dist_x, dist_y: DOUBLE)
			-- Resize `an_element' by moving the side indicated
			-- by `is_left', `is_right', `is_bottom' and `is_top'
			-- the distance indicated by `dist_x', `dist_y'.
			-- Set `is_resized' if side was moved and
			-- leave it alone if no resize
		local
			l_x, l_y: DOUBLE
		do
			l_x := grid.rounded_edge (dist_x)
			l_y := grid.rounded_edge (dist_y)
			if l_x /= 0.0 or l_y /= 0.0 then
				is_resized := True
				resize_amount.make_xy (l_x, l_y)
				an_element.resize_by (resize_amount, is_left, is_right, is_bottom, is_top)
				change_list.extend (create {RESIZE_CHANGE}.make (an_element, resize_amount.x, resize_amount.y, is_left, is_right, is_bottom, is_top))
			end
		end
	
feature {NONE} -- Implementation

	resize_amount: POINT
			-- Temporary to avoid object creation

	is_resized: BOOLEAN
			-- Has object been resized yet?

end
