
note
	
	description: "Object that is an element of a view"
	author: "David Hollenberg"
	date: "July 25, 2006"

deferred class VIEW_ELEMENT
inherit
	VIEW_GRAPHICAL_ELEMENT

feature -- Properties

	has_sub_elements: BOOLEAN
			-- Is element comprised of multiple sub-elements?
		deferred
		end

	sub_elements: LIST [like anchored_view_sub_element]
			-- Sub-elements of `Current'
		require
			has_sub_elements: has_sub_elements
		deferred
		end

	anchored_view_sub_element: VIEW_SUB_ELEMENT
			-- Type of sub-elements in view
		do
		ensure
			result_void: Result = Void
		end

	is_selectable: BOOLEAN
			-- May object be selected?
		do
		end

	is_rotatable: BOOLEAN
			-- May object currently have its orientation changed?
		do
		end

	rotation: INTEGER
			-- Rotation of object in degrees clockwise
		do
		end

	is_rotated_sideways: BOOLEAN
			-- Is object rotated by 90 degrees clockwise or
			-- 90 degrees counterclockwise?
		do
		end

	is_duplicatable: BOOLEAN
			-- May object currently be duplicated?
		do
		end

	is_deletable: BOOLEAN
			-- May object currently be deleted?
		do
		end

	is_fixed_in_x: BOOLEAN
			-- Is object temporarily fixed in the X dimension
			-- so that it cannot move in that dimension
			-- for horizontal scanning purposes?
			-- Returns true if object is locked in X
		do
		end

	is_fixed_in_y: BOOLEAN
			-- Is object temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension
			-- for vertical scanning purposes?
			-- Returns true if object is locked in X
		do
		end

	is_fixed_but_not_locked_in_x: BOOLEAN
			-- Is object temporarily fixed in the X dimension
			-- so that it cannot move in that dimension
			-- for horizontal scanning purposes, but not locked
			-- in X?
		do
		end

	is_fixed_but_not_locked_in_y: BOOLEAN
			-- Is object temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension
			-- for vertical scanning purposes, but not locked
			-- in Y?
		do
		end

	can_be_fixed_in_x: BOOLEAN
			-- Can object be temporarily fixed in the X dimension?
			-- so that it cannot move in that dimension for
			-- horizontal scanning purposes?
		do
		end

	can_be_fixed_in_y: BOOLEAN
			-- Can object be temporarily fixed in the Y dimension
			-- so that it cannot move in that dimension for
			-- vertical scanning purposes?
		do
		end

	is_movable_in_x: BOOLEAN
			-- May object currently be moved in the X dimension?
		do
		end

	is_movable_in_y: BOOLEAN
			-- May object currently be moved in the Y dimension?
		do
		end

	is_locked_in_x: BOOLEAN
			-- May object currently be not moved in the X
			-- dimension (is it locked in position)?
		do
			Result := not is_movable_in_x
		end

	is_locked_in_y: BOOLEAN
			-- May object currently be not moved in the Y
			-- dimension (is it locked in position)?
		do
			Result := not is_movable_in_y
		end

	is_stationary_in_x: BOOLEAN
			-- Can object never be moved in the X dimension?
		do
		end

	is_stationary_in_y: BOOLEAN
			-- Can object never be moved in the Y dimension?
		do
		end

	is_stationary_orientation: BOOLEAN
			-- Can orientation of object never be changed?
		do
		end

	is_never_duplicatable: BOOLEAN
			-- Can object never be duplicated?
		do
		end

	is_never_deletable: BOOLEAN
			-- Can object never be deleted?
		do
		end

	is_fixed_in_x_changed: BOOLEAN
			-- Did last call to `set_is_fixed_in_x' change the
			-- value of `is_fixed_in_x'?

	is_fixed_in_y_changed: BOOLEAN
			-- Did last call to `set_is_fixed_in_y' change teh
			-- value of `is_fixed_in_y'?

	is_movable_in_x_changed: BOOLEAN
			-- Did last call to `set_is_movable_in_x' change the
			-- value of `is_movable_in_x'?

	is_movable_in_y_changed: BOOLEAN
			-- Did last call to `set_is_movable_in_y' change the
			-- value of `is_movable_in_y'?

	is_rotatable_changed: BOOLEAN
			-- Did last call to `set_is_rotatable' change the
			-- value of `is_rotatable'?

	is_duplicatable_changed: BOOLEAN
			-- Did last call to `set_is_duplicatable' change the
			-- value of `is_duplicatable'?

	is_deletable_changed: BOOLEAN
			-- Did last call to `set_is_deletable' change the
			-- value of `is_deletable'?

	is_non_dup_lockable_property_changed: BOOLEAN
			-- Were any properties of element which are lockable
			-- changed (excluding duplication)?
		do
			Result := is_fixed_in_x_changed
				or is_fixed_in_y_changed
				or is_movable_in_x_changed
				or is_movable_in_y_changed
				or is_rotatable_changed
				or is_deletable_changed
		end

	is_lockable_property_changed: BOOLEAN
			-- Were any properties of element which are lockable
			-- changed?
		do
			Result := is_fixed_in_x_changed
				or is_fixed_in_y_changed
				or is_movable_in_x_changed
				or is_movable_in_y_changed
				or is_rotatable_changed
				or is_duplicatable_changed
				or is_deletable_changed
		end

	is_displayed: BOOLEAN
			-- Is element currently displayed?  May not be
			-- visible if displayed if zoomed in on area
			-- that does not include element
		deferred
		end

feature -- Modification

	fix_in_x
			-- Fix `Current' in X dimension, if possible
		do
			if can_be_fixed_in_x then
				set_is_fixed_in_x (True)
			else
				is_fixed_in_x_changed := False
			end
		end

	fix_in_y
			-- Fix `Current' in Y dimension, if possible
		do
			if can_be_fixed_in_y then
				set_is_fixed_in_y (True)
			else
				is_fixed_in_x_changed := False
			end
		end

	fix_in_x_and_y
			-- Fix `Current' in X and Y dimensions, if possible
		do
			fix_in_x
			fix_in_y
		end

	unfix_in_x
			-- Unfix `Current' in X dimension, if possible
		do
			if not is_stationary_in_x then
				set_is_fixed_in_x (False)
			else
				is_fixed_in_x_changed := False
			end
		end

	unfix_in_y
			-- Unfix `Current' in Y dimension, if possible
		do
			if not is_stationary_in_y then
				set_is_fixed_in_y (False)
			else
				is_fixed_in_y_changed := False
			end
		end

	unfix_in_x_and_y
			-- Unfix `Current' in X and Y dimensions, if possible
		do
			unfix_in_x
			unfix_in_y
		end

	lock_in_x
			-- Lock `Current' in X dimension, if possible
		do
			if not is_stationary_in_x then
				set_is_movable_in_x (False)
			else
				is_movable_in_x_changed := False
			end
		end

	lock_in_y
			-- Lock `Current' in Y dimension, if possible
		do
			if not is_stationary_in_y then
				set_is_movable_in_y (False)
			else
				is_movable_in_y_changed := False
			end
		end

	lock_in_x_and_y
			-- Lock `Current' in X and Y dimensions, if possible
		do
			lock_in_x
			lock_in_y
		end

	lock_orientation
			-- Lock orientation of `Current', if possible
		do
			if not is_stationary_orientation then
				set_is_rotatable (False)
			else
				is_rotatable_changed := False
			end
		end

	lock_duplication
			-- Lock `Current' to prevent it from being duplicated
		do
			if not is_never_duplicatable then
				set_is_duplicatable (False)
			else
				is_duplicatable_changed := False
			end
		end

	lock_deletion
			-- Lock `Current' to prevent it from being deleted
		do
			if not is_never_deletable then
				set_is_deletable (False)
			else
				is_deletable_changed := False
			end
		end

	lock_almost_completely
			-- Lock `Current' almost completely (everything but
			-- duplication)
		do
			unfix_in_x_and_y
			lock_in_x_and_y
			lock_orientation
			lock_deletion
		end

	lock_completely
			-- Lock `Current' completely
		do
			unfix_in_x_and_y
			lock_in_x_and_y
			lock_orientation
			lock_duplication
			lock_deletion
		end

	unlock_in_x
			-- Unlock `Current' in X dimension, if possible
		do
			if not is_stationary_in_x then
				set_is_movable_in_x (True)
			else
				is_movable_in_x_changed := False
			end
		end

	unlock_in_y
			-- Unlock `Current' in Y dimension, if possible
		do
			if not is_stationary_in_y then
				set_is_movable_in_y (True)
			else
				is_movable_in_y_changed := False
			end
		end

	unlock_in_x_and_y
			-- Lock `Current' in X and Y dimensions, if possible
		do
			unlock_in_x
			unlock_in_y
		end

	unlock_orientation
			-- Unlock orientation of `Current', if possible
		do
			if not is_stationary_orientation then
				set_is_rotatable (True)
			else
				is_rotatable_changed := False
			end
		end

	unlock_duplication
			-- Unlock `Current' to allow it to be duplicated,
			-- if possible
		do
			if not is_never_duplicatable then
				set_is_duplicatable (True)
			else
				is_duplicatable_changed := False
			end
		end

	unlock_deletion
			-- Unlock `Current' to allow it to be deleted,
			-- if possible
		do
			if not is_never_deletable then
				set_is_deletable (True)
			else
				is_deletable_changed := False
			end
		end

	unlock_almost_completely
			-- Unlock `Current' almost completely (everything
			-- except duplication)
		do
			is_fixed_in_x_changed := False
			is_fixed_in_y_changed := False
			unlock_in_x_and_y
			unlock_orientation
			unlock_deletion
		end

	unlock_completely
			-- Unlock `Current' completely
		do
			is_fixed_in_x_changed := False
			is_fixed_in_y_changed := False
			unlock_in_x_and_y
			unlock_orientation
			unlock_duplication
			unlock_deletion
		end

	set_rotation (n: INTEGER)
			-- Set rotation of `Current' to `n' degrees clockwise,
			-- if possible
		do
			if is_rotatable then
			end
		end

	toggle_rotation
			-- Toggle rotation of `Current', if possible
		do
			if is_rotatable then
			end
		end

	rotate_90_clockwise
			-- Rotate `Current' 90 degrees clockwise, if possible
		do
			if is_rotatable then
			end
		end

	rotate_90_counterclockwise
			-- Rotate `Current' 90 degrees
			-- counterclockwise, if possible
		do
			if is_rotatable then
			end
		end

	move_by (a_distance: CONST_POINT)
			-- Move `Current' by `a_distance'
		require
			a_distance_not_void: a_distance /= Void
		do
		end

	move_by_without_adjustment (a_distance: CONST_POINT)
			-- Move `Current' by `a_distance'
			-- but do not adjust the center to
			-- keep the edges on grid
		require
			a_distance_not_void: a_distance /= Void
		do
		end

	set_is_fixed_in_x (b: BOOLEAN)
			-- Set `is_fixed_in_x' to `b'
		require
			can_be_fixed_in_x: b implies can_be_fixed_in_x
			can_be_unfixed_in_x: not b implies not is_stationary_in_x
		do
			is_fixed_in_x_changed := is_fixed_but_not_locked_in_x /= b
		ensure
			fixed_in_x_set: is_fixed_in_x = b
		end

	set_is_fixed_in_y (b: BOOLEAN)
			-- Set `is_fixed_in_y' to `b'
		require
			can_be_fixed_in_y: b implies can_be_fixed_in_y
			can_be_unfixed_in_y: not b implies not is_stationary_in_y
		do
			is_fixed_in_y_changed := is_fixed_but_not_locked_in_y /= b
		ensure
			fixed_in_y_set: is_fixed_in_y = b
		end

	set_is_movable_in_x (b: BOOLEAN)
			-- Set `is_movable_in_x' to `b'
		require
			not_stationary_in_x: not is_stationary_in_x
		do
			is_movable_in_x_changed := is_movable_in_x /= b
		ensure
			movable_in_x_set: is_movable_in_x = b
		end

	set_is_movable_in_y (b: BOOLEAN)
			-- Set `is_movable_in_y' to `b'
		require
			not_stationary_in_y: not is_stationary_in_y
		do
			is_movable_in_y_changed := is_movable_in_y /= b
		ensure
			movable_in_y_set: is_movable_in_y = b
		end

	set_is_rotatable (b: BOOLEAN)
			-- Set `is_rotatable' to `b'
		require
			not_stationary_orientation: not is_stationary_orientation
		do
			is_rotatable_changed := is_rotatable /= b
		ensure
			is_rotatable_set: is_rotatable = b
		end

	set_is_duplicatable (b: BOOLEAN)
			-- Set `is_duplicatable' to `b'
		require
			may_be_duplicatable: not is_never_duplicatable
		do
			is_duplicatable_changed := is_duplicatable /= b
		ensure
			is_duplicatable_set: is_duplicatable = b
		end

	set_is_deletable (b: BOOLEAN)
			-- Set `is_deletable' to `b'
		require
			may_be_deletable: not is_never_deletable
		do
			is_deletable_changed := is_deletable /= b
		ensure
			is_deletable_set: is_deletable = b
		end

invariant
	drawer_not_void: drawer /= Void
	view_not_void: view /= Void

end

